# Generated by Django 4.1.8 on 2023-04-19 05:42

import uuid

import colorfield.fields
import django.db.models.deletion
import django_better_admin_arrayfield.models.fields
import wbcore.markdown.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("reversion", "0001_squashed_0004_auto_20160611_1202"),
        ("auth", "0012_alter_user_first_name_max_length"),
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="FrontendUserConfiguration",
            fields=[
                ("id", models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ("config", models.JSONField(blank=True, default=dict, null=True)),
                (
                    "parent_configuration",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="child_configurations",
                        to="wbcore.frontenduserconfiguration",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="configurations",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Frontend User Settings",
                "verbose_name_plural": "Frontend User Settings",
            },
        ),
        migrations.CreateModel(
            name="Notification",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("title", models.CharField(max_length=512)),
                ("message", models.TextField(blank=True, null=True)),
                ("buttons", models.JSONField(blank=True, default=list, null=True)),
                ("timestamp_created", models.DateTimeField(auto_now_add=True, verbose_name="Created")),
                ("timestamp_received", models.DateTimeField(blank=True, null=True, verbose_name="Received")),
                ("timestamp_read", models.DateTimeField(blank=True, null=True, verbose_name="Read")),
                ("timestamp_mailed", models.DateTimeField(blank=True, null=True, verbose_name="Mailed")),
                (
                    "send_type",
                    models.CharField(
                        choices=[("system", "System"), ("mail", "Mail"), ("system_and_mail", "System and Mail")],
                        default="system",
                        max_length=32,
                    ),
                ),
                (
                    "recipient",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="notifications",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                ("endpoint", models.URLField(blank=True, max_length=2048, null=True)),
            ],
        ),
        migrations.CreateModel(
            name="Tag",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("title", models.CharField(max_length=255, unique=True)),
                ("color", models.CharField(default="#D3D3D3", max_length=7)),
                ("slug", models.CharField(blank=True, max_length=255, null=True)),
            ],
            options={
                "verbose_name": "Tag",
                "verbose_name_plural": "Tags",
            },
        ),
        migrations.CreateModel(
            name="RevisionAbstract",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("reversion.revision",),
        ),
        migrations.CreateModel(
            name="VersionAbstract",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("reversion.version",),
        ),
        migrations.CreateModel(
            name="Asset",
            fields=[
                ("id", models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ("file", models.FileField(upload_to=wbcore.markdown.models.upload_to)),
                ("content_type", models.CharField(blank=True, max_length=32, null=True)),
                ("file_url_name", models.CharField(blank=True, max_length=1024, null=True)),
            ],
            options={
                "verbose_name": "Asset",
                "verbose_name_plural": "Assets",
            },
        ),
        migrations.CreateModel(
            name="BigUserObjectPermission",
            fields=[
                ("object_pk", models.CharField(max_length=255, verbose_name="object ID")),
                ("id", models.BigAutoField(editable=False, primary_key=True, serialize=False, unique=True)),
                (
                    "content_type",
                    models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="contenttypes.contenttype"),
                ),
                ("permission", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="auth.permission")),
                ("user", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.AddIndex(
            model_name="biguserobjectpermission",
            index=models.Index(fields=["content_type", "object_pk"], name="wbcore_big_content_6fb9b7_idx"),
        ),
        migrations.AddIndex(
            model_name="biguserobjectpermission",
            index=models.Index(fields=["content_type", "object_pk", "user"], name="wbcore_big_content_bcec08_idx"),
        ),
        migrations.AlterUniqueTogether(
            name="biguserobjectpermission",
            unique_together={("user", "permission", "object_pk")},
        ),
        migrations.CreateModel(
            name="BigGroupObjectPermission",
            fields=[
                ("object_pk", models.CharField(max_length=255, verbose_name="object ID")),
                ("id", models.BigAutoField(editable=False, primary_key=True, serialize=False, unique=True)),
                (
                    "content_type",
                    models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="contenttypes.contenttype"),
                ),
                ("group", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="auth.group")),
                ("permission", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="auth.permission")),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.AddIndex(
            model_name="biggroupobjectpermission",
            index=models.Index(fields=["content_type", "object_pk"], name="wbcore_big_content_9cf161_idx"),
        ),
        migrations.AddIndex(
            model_name="biggroupobjectpermission",
            index=models.Index(fields=["content_type", "object_pk", "group"], name="wbcore_big_content_c9791f_idx"),
        ),
        migrations.AlterUniqueTogether(
            name="biggroupobjectpermission",
            unique_together={("group", "permission", "object_pk")},
        ),
        migrations.CreateModel(
            name="ReleaseNote",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("version", models.CharField(help_text="The version/identifier of the release", max_length=18)),
                ("release_date", models.DateField(help_text="The date, when this new version was released")),
                ("module", models.CharField(help_text="The workbench-module of the release", max_length=255)),
                ("notes", models.TextField(default="", help_text="What's new? What's improved? What's fixed?")),
                ("read_by", models.ManyToManyField(related_name="read_patch_notes", to=settings.AUTH_USER_MODEL)),
            ],
            options={
                "verbose_name": "Release Note",
                "verbose_name_plural": "Release Notes",
            },
        ),
        migrations.AddConstraint(
            model_name="releasenote",
            constraint=models.UniqueConstraint(
                fields=("version", "module"), name="release_note_unique_version_module"
            ),
        ),
        migrations.AddField(
            model_name="releasenote",
            name="summary",
            field=models.CharField(default="abc", help_text="A brief summary of the release", max_length=512),
            preserve_default=False,
        ),
        migrations.DeleteModel(
            name="RevisionAbstract",
        ),
        migrations.DeleteModel(
            name="VersionAbstract",
        ),
        migrations.DeleteModel(
            name="Tag",
        ),
        migrations.AlterField(
            model_name="asset",
            name="file",
            field=models.FileField(max_length=256, upload_to=wbcore.markdown.models.upload_to),
        ),
        migrations.AddField(
            model_name="biggroupobjectpermission",
            name="editable",
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name="biguserobjectpermission",
            name="editable",
            field=models.BooleanField(default=True),
        ),
        migrations.CreateModel(
            name="PermissionTestModel",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "permission_type",
                    models.CharField(
                        choices=[("INTERNAL", "Internal"), ("PUBLIC", "Public"), ("PRIVATE", "Private")],
                        default="PRIVATE",
                        max_length=8,
                    ),
                ),
            ],
            options={
                "abstract": False,
                "managed": False,
                "default_permissions": ("add", "change", "delete", "view", "administrate"),
            },
        ),
        migrations.CreateModel(
            name="ColorGradient",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("title", models.CharField(default="", max_length=128)),
                (
                    "colors",
                    django_better_admin_arrayfield.models.fields.ArrayField(
                        base_field=colorfield.fields.ColorField(
                            default="#FF0000", image_field=None, max_length=18, samples=None
                        ),
                        default=list,
                        size=None,
                    ),
                ),
            ],
        ),
        migrations.AlterField(
            model_name="releasenote",
            name="module",
            field=models.CharField(help_text="The workbench module of the release", max_length=255),
        ),
        migrations.AlterField(
            model_name="releasenote",
            name="release_date",
            field=models.DateField(help_text="The date when this new version was released"),
        ),
        migrations.AlterField(
            model_name="releasenote",
            name="read_by",
            field=models.ManyToManyField(
                db_table="bridger_releasenote_read_by", related_name="read_patch_notes", to=settings.AUTH_USER_MODEL
            ),
        ),
        migrations.AlterModelTable(
            name="asset",
            table="bridger_asset",
        ),
        migrations.AlterModelTable(
            name="biggroupobjectpermission",
            table="bridger_biggroupobjectpermission",
        ),
        migrations.AlterModelTable(
            name="biguserobjectpermission",
            table="bridger_biguserobjectpermission",
        ),
        migrations.AlterModelTable(
            name="colorgradient",
            table="bridger_colorgradient",
        ),
        migrations.AlterModelTable(
            name="frontenduserconfiguration",
            table="bridger_frontenduserconfiguration",
        ),
        migrations.AlterModelTable(
            name="notification",
            table="bridger_notification",
        ),
        migrations.AlterModelTable(
            name="releasenote",
            table="bridger_releasenote",
        ),
        migrations.RenameIndex(
            model_name="biggroupobjectpermission",
            new_name="bridger_big_content_9cf161_idx",
            old_name="wbcore_big_content_9cf161_idx",
        ),
        migrations.RenameIndex(
            model_name="biggroupobjectpermission",
            new_name="bridger_big_content_c9791f_idx",
            old_name="wbcore_big_content_c9791f_idx",
        ),
        migrations.RenameIndex(
            model_name="biguserobjectpermission",
            new_name="bridger_big_content_6fb9b7_idx",
            old_name="wbcore_big_content_6fb9b7_idx",
        ),
        migrations.RenameIndex(
            model_name="biguserobjectpermission",
            new_name="bridger_big_content_bcec08_idx",
            old_name="wbcore_big_content_bcec08_idx",
        ),
        migrations.AlterField(
            model_name="notification",
            name="endpoint",
            field=models.CharField(blank=True, max_length=2048, null=True),
        ),
        migrations.AddField(
            model_name="notification",
            name="share_endpoint_as_widget",
            field=models.BooleanField(
                default=True,
                help_text="If True, it will share the endpoint as widget_endpoint. Otherwise, sends the endpoint as url",
            ),
        ),
        migrations.DeleteModel(
            name="PermissionTestModel",
        ),
        migrations.CreateModel(
            name="Preset",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("title", models.CharField(max_length=64)),
                ("display_identifier", models.CharField(max_length=512)),
                ("display", models.JSONField(blank=True, null=True)),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="presets",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="AppliedPreset",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("display_identifier_path", models.CharField(max_length=1024)),
                ("display", models.JSONField(blank=True, null=True)),
                (
                    "preset",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="applied_presets",
                        to="wbcore.preset",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="applied_presets",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



from django.db import models

from wbcore.models.orderable import OrderableModel


class ForeignKeyClass(models.Model):  # noqa
    @classmethod
    def get_endpoint_basename(cls) -> str:
        return "wbcore:foreignkeyclass"

    @classmethod
    def get_representation_value_key(cls) -> str:
        return "id"


class OrderableChild(OrderableModel):
    PARTITION_BY = "partition_fk"

    partition_fk = models.ForeignKey(
        to=ForeignKeyClass,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )

    @classmethod
    def get_endpoint_basename(cls) -> str:
        return "wbcore:orderablechild"

    @classmethod
    def get_representation_value_key(cls) -> str:
        return "id"



from django.db.models import QuerySet
from django.utils.translation import gettext_lazy as _

from wbcore import filters
from wbcore.contrib.example_app.models import Player, SportPerson, Stadium, Team
from wbcore.contrib.geography.models import Geography


class TeamStadiumFilter(filters.FilterSet):
    founded_date__gte = filters.DateFilter(
        label=_("Founded Date"),
        lookup_expr="gte",
        field_name="founded_date",
    )
    founded_date__lte = filters.DateFilter(
        label=_("Founded Date"),
        lookup_expr="lte",
        field_name="founded_date",
    )
    coach = filters.ModelMultipleChoiceFilter(
        label=_("Coaches"),
        queryset=SportPerson.objects.all(),
        endpoint=SportPerson.get_representation_endpoint(),
        value_key=SportPerson.get_representation_value_key(),
        label_key=SportPerson.get_representation_label_key(),
    )
    current_players = filters.ModelChoiceFilter(
        label=_("Current Player"),
        queryset=Player.objects.all(),
        endpoint=Player.get_representation_endpoint(),
        value_key=Player.get_representation_value_key(),
        label_key=Player.get_representation_label_key(),
    )
    former_players = filters.ModelChoiceFilter(
        label=_("Former Player"),
        queryset=Player.objects.all(),
        endpoint=Player.get_representation_endpoint(),
        value_key=Player.get_representation_value_key(),
        label_key=Player.get_representation_label_key(),
    )
    city = filters.ModelMultipleChoiceFilter(
        label=_("Cities"),
        queryset=Geography.cities.all(),
        endpoint=Geography.get_representation_endpoint(),
        value_key=Geography.get_representation_value_key(),
        label_key=Geography.get_representation_label_key(),
        filter_params={"level": 3},
    )

    class Meta:
        model = Team
        fields = {
            "name": ["exact", "icontains"],
        }


class TeamFilter(TeamStadiumFilter):
    opponent = filters.ModelChoiceFilter(
        label=_("Opponent"),
        queryset=Team.objects.all(),
        endpoint=Team.get_representation_endpoint(),
        value_key=Team.get_representation_value_key(),
        label_key=Team.get_representation_label_key(),
        method="filter_opponent",
    )

    def filter_opponent(self, queryset: QuerySet[Team], name, value: Team) -> QuerySet[Team]:
        if value:
            return queryset.exclude(id=value.id)
        return queryset

    home_stadium = filters.ModelMultipleChoiceFilter(
        label=_("Stadiums"),
        queryset=Stadium.objects.all(),
        endpoint=Stadium.get_representation_endpoint(),
        value_key=Stadium.get_representation_value_key(),
        label_key=Stadium.get_representation_label_key(),
    )



from django.core.exceptions import FieldError
from django.db import models


class PandasFilterSetMixin:
    def filter_queryset(self, queryset):
        """
        Filter the queryset with the underlying form's `cleaned_data`. You must
        call `is_valid()` or `errors` before calling this method.

        This method should be overridden if additional filtering needs to be
        applied to the queryset before it is cached.
        """
        for name, value in self.form.cleaned_data.items():
            try:
                queryset = self.filters[name].filter(queryset, value)
            except FieldError:
                pass
            if not isinstance(queryset, models.QuerySet):
                raise AssertionError(
                    "Expected '%s.%s' to return a QuerySet, but got a %s instead."
                    % (
                        type(self).__name__,
                        name,
                        type(queryset).__name__,
                    )
                )
        return queryset



from django.utils.translation import gettext as _

from wbcore.menus import ItemPermission, MenuItem
from wbcore.permissions.shortcuts import is_internal_user

STARTSTEP_MENUITEM = MenuItem(
    label=_("Start Step"),
    endpoint="wbcore:workflow:startstep-list",
    permission=ItemPermission(
        method=lambda request: is_internal_user(request.user), permissions=["workflow.view_startstep"]
    ),
    add=MenuItem(
        label=_("Create Start Step"),
        endpoint="wbcore:workflow:startstep-list",
        permission=ItemPermission(
            method=lambda request: is_internal_user(request.user), permissions=["workflow.add_startstep"]
        ),
    ),
)

USERSTEP_MENUITEM = MenuItem(
    label=_("User Step"),
    endpoint="wbcore:workflow:userstep-list",
    permission=ItemPermission(
        method=lambda request: is_internal_user(request.user), permissions=["workflow.view_userstep"]
    ),
    add=MenuItem(
        label=_("Create User Step"),
        endpoint="wbcore:workflow:userstep-list",
        permission=ItemPermission(
            method=lambda request: is_internal_user(request.user), permissions=["workflow.add_userstep"]
        ),
    ),
)

DECISIONSTEP_MENUITEM = MenuItem(
    label=_("Decision Step"),
    endpoint="wbcore:workflow:decisionstep-list",
    permission=ItemPermission(
        method=lambda request: is_internal_user(request.user), permissions=["workflow.view_decisionstep"]
    ),
    add=MenuItem(
        label=_("Create Decision Step"),
        endpoint="wbcore:workflow:decisionstep-list",
        permission=ItemPermission(
            method=lambda request: is_internal_user(request.user), permissions=["workflow.add_decisionstep"]
        ),
    ),
)

SPLITSTEP_MENUITEM = MenuItem(
    label=_("Split Step"),
    endpoint="wbcore:workflow:splitstep-list",
    permission=ItemPermission(
        method=lambda request: is_internal_user(request.user), permissions=["workflow.view_splitstep"]
    ),
    add=MenuItem(
        label=_("Create Split Step"),
        endpoint="wbcore:workflow:splitstep-list",
        permission=ItemPermission(
            method=lambda request: is_internal_user(request.user), permissions=["workflow.add_splitstep"]
        ),
    ),
)

JOINSTEP_MENUITEM = MenuItem(
    label=_("Join Step"),
    endpoint="wbcore:workflow:joinstep-list",
    permission=ItemPermission(
        method=lambda request: is_internal_user(request.user), permissions=["workflow.view_joinstep"]
    ),
    add=MenuItem(
        label=_("Create Join Step"),
        endpoint="wbcore:workflow:joinstep-list",
        permission=ItemPermission(
            method=lambda request: is_internal_user(request.user), permissions=["workflow.add_joinstep"]
        ),
    ),
)

SCRIPTSTEP_MENUITEM = MenuItem(
    label=_("Script Step"),
    endpoint="wbcore:workflow:scriptstep-list",
    permission=ItemPermission(
        method=lambda request: is_internal_user(request.user), permissions=["workflow.view_scriptstep"]
    ),
    add=MenuItem(
        label=_("Create Script Step"),
        endpoint="wbcore:workflow:scriptstep-list",
        permission=ItemPermission(
            method=lambda request: is_internal_user(request.user), permissions=["workflow.add_scriptstep"]
        ),
    ),
)

EMAILSTEP_MENUITEM = MenuItem(
    label=_("Email Step"),
    endpoint="wbcore:workflow:emailstep-list",
    permission=ItemPermission(
        method=lambda request: is_internal_user(request.user), permissions=["workflow.view_emailstep"]
    ),
    add=MenuItem(
        label=_("Create Email Step"),
        endpoint="wbcore:workflow:emailstep-list",
        permission=ItemPermission(
            method=lambda request: is_internal_user(request.user), permissions=["workflow.add_emailstep"]
        ),
    ),
)

FINISHSTEP_MENUITEM = MenuItem(
    label=_("Finish Step"),
    endpoint="wbcore:workflow:finishstep-list",
    permission=ItemPermission(
        method=lambda request: is_internal_user(request.user), permissions=["workflow.view_finishstep"]
    ),
    add=MenuItem(
        label=_("Create Finish Step"),
        endpoint="wbcore:workflow:finishstep-list",
        permission=ItemPermission(
            method=lambda request: is_internal_user(request.user), permissions=["workflow.add_finishstep"]
        ),
    ),
)



from django.utils.translation import gettext as _

from wbcore.menus import ItemPermission, MenuItem
from wbcore.permissions.shortcuts import is_internal_user

WORKFLOW_MENUITEM = MenuItem(
    label=_("Workflow"),
    endpoint="wbcore:workflow:workflow-list",
    permission=ItemPermission(
        method=lambda request: is_internal_user(request.user), permissions=["workflow.view_workflow"]
    ),
    add=MenuItem(
        label=_("Create Workflow"),
        endpoint="wbcore:workflow:workflow-list",
        permission=ItemPermission(
            method=lambda request: is_internal_user(request.user), permissions=["workflow.add_workflow"]
        ),
    ),
)



from contextlib import suppress

from django.contrib.contenttypes.models import ContentType


def get_ancestors_content_type(content_type):
    """
    Utility function to gather the ascendant models content types
    Returns a generator
    class A:
        ...

    Class B(A):
        ...

    get_ascendant_content_types(ContentType.objects.get_for_model(B)) = [
        ContentType.objects.get_for_model(A),
        ContentType.objects.get_for_model(B)
    ]
    """
    for _class in content_type.model_class().__mro__:
        with suppress(Exception):
            ancestor_content_type = ContentType.objects.get_for_model(_class)
            if ancestor_content_type.model_class():
                yield ancestor_content_type


def get_view_content_type_id(view):
    model = view.get_queryset().model
    return ContentType.objects.get_for_model(model).id



from rest_framework.fields import Field

from wbcore.serializers import PrimaryKeyRelatedField
from wbcore.serializers.fields.mixins import WBCoreSerializerFieldMixin
from wbcore.serializers.fields.related import WBCoreManyRelatedField

# from tests.models import ModelTest


class TestWBCoreManyRelatedField:
    def setup_method(self):
        class TestSubClass(WBCoreSerializerFieldMixin, Field):
            field_type = "field_type"

        self.TestSubClass = TestSubClass
        self.field = WBCoreManyRelatedField

    def test_not_none(self):
        assert self.field(child_relation=self.TestSubClass()) is not None

    def test_required_allowed_null(self):
        field = self.field(child_relation=self.TestSubClass(), required=False)
        assert field.allow_null

    def test_validation_allow_null(self):
        field = self.field(child_relation=self.TestSubClass(), required=False)
        validated_data = field.run_validation(None)

        assert validated_data == []

    def test_get_representation(self):
        field = self.field(child_relation=self.TestSubClass())
        assert field.get_representation(None, "field_name")[1]["multiple"]


class TestPrimaryKeyRelatedField:
    def setup_method(self):
        self.field = PrimaryKeyRelatedField

    def test_is_not_none_read_only(self):
        assert self.field(read_only=True) is not None

    # def test_is_not_none_queryset(self):
    #     assert self.field(queryset=ModelTest.objects.all()) is not None

    # def test_many_is_not_none(self):
    #     field = self.field.many_init(required=True, queryset=ModelTest.objects.all())
    #     assert field is not None
    #     assert isinstance(field, WBCoreManyRelatedField)

    # def test_get_representation(self):
    #     assert self.field(queryset=ModelTest.objects.all()).get_representation(None, "field_name") == {
    #         "key": "field_name",
    #         "label": None,
    #         "type": WBCoreType.SELECT.value,
    #         "required": True,
    #         "read_only": False,
    #         "decorators": [],
    #         "depends_on": [],
    #
    #     }

    # def test_get_representation_multiple(self):
    #     assert self.field.many_init(queryset=ModelTest.objects.all()).get_representation(None, "field_name") == {
    #         "key": "field_name",
    #         "label": "",
    #         "type": WBCoreType.SELECT.value,
    #         "required": True,
    #         "read_only": False,
    #         "multiple": True,
    #         "decorators": [],
    #         "depends_on": [],
    #
    #     }

    # @pytest.mark.django_db
    # def test_run_validation(self, model_test):
    #     validated_data = self.field(required=False, queryset=ModelTest.objects.all()).run_validation(
    #         data=model_test.pk
    #     )
    #     assert validated_data == model_test



from contextlib import suppress

from django.utils.text import slugify
from rest_framework.request import Request
from rest_framework.views import View

from wbcore.metadata.configs.display.instance_display.display import Display
from wbcore.metadata.configs.display.list_display import ListDisplay
from wbcore.metadata.configs.display.windows import Window

from ..base import WBCoreViewConfig
from .models import AppliedPreset


class DisplayViewConfig(WBCoreViewConfig):
    metadata_key = "display"
    config_class_attribute = "display_config_class"

    def __init__(self, view: View, request: Request, instance: bool | None = None):
        self.tooltip = request.GET.get("tooltip", None) == "true"
        self.inline = request.GET.get("inline", None) == "true"
        super().__init__(view, request, instance)

    def get_window(self) -> Window | None:
        return None

    def get_instance_display(self) -> Display | None:
        return None

    def get_list_display(self) -> ListDisplay | None:
        return None

    def get_preview_display(self) -> Display | None:
        return None

    def get_metadata(self) -> dict:
        display = dict()
        instance_display = self.get_instance_display()
        if isinstance(instance_display, Display):
            display["instance"] = instance_display.serialize(view_config=self, view=self.view, request=self.request)
        elif instance_display:
            display["instance"] = list(instance_display)
        else:
            display["instance"] = Display(pages=[]).serialize()

        if not self.instance:
            list_display = self.get_list_display()
            if isinstance(list_display, Display):
                display["list"] = list_display.serialize()
            else:
                display["list"] = dict(list_display or {})

        if window := self.get_window():
            display["window"] = window.serialize()

        display_identifier_path = self.view.display_identifier_config_class(
            self.view, self.request, self.instance
        ).display_identifier_path()

        with suppress(AppliedPreset.DoesNotExist):
            display["preset"] = AppliedPreset.objects.get(
                user=self.request.user, display_identifier_path=display_identifier_path
            ).display

        return display


class DisplayIdentifierViewConfig(WBCoreViewConfig):
    metadata_key = "display_identifier"
    config_class_attribute = "display_identifier_config_class"

    def display_identifier_path(self) -> str:
        display = self.view.display_config_class
        slugified_display_module = slugify(display.__module__.replace(".", "-"))
        slugified_display_class = slugify(display.__name__)
        display_identifier_path = f"{slugified_display_module}-{slugified_display_class}"

        # We get the path from the header (if it exists, only for nested tables inside forms) and then join it
        # with the current display identifier. If there is an applied preset for this user - we return it.
        if inline_path := self.request.META.get("HTTP_WB_DISPLAY_IDENTIFIER", None):
            display_identifier_path = f"{inline_path}.{display_identifier_path}"
        return display_identifier_path

    def get_metadata(self):
        return self.display_identifier_path()



import re

from django.conf import settings
from django.db.models import Q
from dynamic_preferences.registries import global_preferences_registry

from ..models import Source


def is_sender_allowed(from_email: str, whitelisted_emails: list[str], admin_mails: list[str]) -> bool:
    if from_email in admin_mails:
        return True
    for whitelisted_email in whitelisted_emails:
        if re.search(whitelisted_email, from_email):
            return True
    return False


def handle_inbound(sender, event, esp_name, **kwargs):
    spam_detected = False
    if (message := event.message) and (from_email := message.from_email.addr_spec) and (subject := message.subject):
        if spam_score := getattr(settings, "WBIMPORT_EXPORT_MAILBACKEND_SPAMSCORE", None):
            spam_detected = message.spam_detected or message.spam_score >= spam_score
        if not spam_detected:
            admin_emails = global_preferences_registry.manager()["io__administrator_mails"].split(";")
            conditions = Q(import_parameters__inbound_address__isnull=True)
            for t in message.to:
                conditions |= Q(import_parameters__inbound_address__contains=t.addr_spec)

            sources = Source.objects.filter(
                conditions
                & Q(data_backend__backend_class_path="wbcore.contrib.io.import_export.backends.mail")
                & Q(is_active=True)
            )
            if s := re.search(r"\[([0-9a-f]{8}-[0-9a-f]{4}-4[0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12})\]", subject):
                sources = sources.filter(uuid=s.group(1))
            for source in sources:
                if is_sender_allowed(from_email, source.import_parameters.get("whitelisted_emails", []), admin_emails):
                    source.trigger_workflow(message=message)



from django.utils.translation import gettext_lazy as _

from wbcore import filters as wb_filters

from ..models import (
    ClientManagerRelationship,
    Entry,
    Person,
    Relationship,
    RelationshipType,
)


class RelationshipEntryFilter(wb_filters.FilterSet):
    relationship_type = wb_filters.ModelMultipleChoiceFilter(
        label=_("Relationship Types"),
        queryset=RelationshipType.objects.all(),
        endpoint=RelationshipType.get_representation_endpoint(),
        value_key=RelationshipType.get_representation_value_key(),
        label_key=RelationshipType.get_representation_label_key(),
    )

    to_entry = wb_filters.ModelMultipleChoiceFilter(
        label=_("Relationship To"),
        queryset=Entry.objects.all(),
        endpoint=Entry.get_representation_endpoint(),
        value_key=Entry.get_representation_value_key(),
        label_key=Entry.get_representation_label_key(),
    )

    class Meta:
        model = Relationship
        fields = {}


class RelationshipFilter(RelationshipEntryFilter):
    from_entry = wb_filters.ModelMultipleChoiceFilter(
        label=_("Relationship From"),
        queryset=Entry.objects.all(),
        endpoint=Entry.get_representation_endpoint(),
        value_key=Entry.get_representation_value_key(),
        label_key=Entry.get_representation_label_key(),
    )


class RelationshipTypeFilter(wb_filters.FilterSet):
    counter_relationship = wb_filters.ModelMultipleChoiceFilter(
        label=_("Counter Relationship Types"),
        queryset=RelationshipType.objects.all(),
        endpoint=RelationshipType.get_representation_endpoint(),
        value_key=RelationshipType.get_representation_value_key(),
        label_key=RelationshipType.get_representation_label_key(),
    )

    class Meta:
        model = RelationshipType
        fields = {"title": ["exact", "icontains"]}


class ClientManagerFilter(wb_filters.FilterSet):
    client = wb_filters.ModelMultipleChoiceFilter(
        label=_("Clients"),
        queryset=Entry.objects.all(),
        endpoint=Entry.get_representation_endpoint(),
        value_key=Entry.get_representation_value_key(),
        label_key=Entry.get_representation_label_key(),
    )
    relationship_manager = wb_filters.ModelMultipleChoiceFilter(
        label=_("Future Relationship Managers"),
        queryset=Person.objects.all(),
        endpoint=Person.get_representation_endpoint(),
        value_key=Person.get_representation_value_key(),
        label_key=Person.get_representation_label_key(),
    )
    status = wb_filters.ChoiceFilter(
        label=_("Status"),
        choices=ClientManagerRelationship.Status.choices,
    )

    def filter_already_in_charge(self, queryset, name, value):
        if value:
            return queryset.filter(relationship_managers__in=value)
        return queryset

    class Meta:
        model = ClientManagerRelationship
        fields = {
            "created": ["exact"],
            "primary": ["exact"],
        }



from django.db import models

from wbcore.contrib.dataloader.dataloaders import DataloaderProxy
from wbcore.contrib.dataloader.models.querysets import EntityQuerySet


class Entity(models.Model):
    """An abstract model representing an entity with associated dataloaders.

    This model serves as a base for entities that store data and have dataloaders
    attached for data retrieval and processing. It includes specialized fields to
    facilitate the use of dataloaders:

    Attributes:
        dl_proxy (DataloaderProxy): A reference to the DataloaderProxy. This should be
            set when using this model to manage the associated dataloaders.
        dl_parameters (models.JSONField): A field to store parameters for various
            dataloaders. It is structured in the format:
            `{"dataloader_name": {"path": "path.to.dataloader", "parameters": {}}}`.
        objects (EntityQuerySet): A custom model manager providing an interface to a
            dataloader proxy. This proxy can be accessed through the 'dl' method on a
            queryset, e.g., `Entity.objects.all().dl`.
    """

    dl_proxy: DataloaderProxy
    dl_parameters = models.JSONField(default=dict, blank=True)
    objects = EntityQuerySet.as_manager()

    class Meta:
        abstract = True



from wbcore.metadata.configs.display.instance_display import operators as op
from wbcore.metadata.configs.display.instance_display.layouts.layouts import Layout
from wbcore.metadata.configs.display.instance_display.pages import Page


class TestPage:
    def test_init(self):
        assert Page(layouts={op.default(): Layout([[]])}) is not None



from django.apps import AppConfig
from django.db.models.signals import post_save
from django.utils.module_loading import autodiscover_modules

from wbcore.signals import add_dynamic_button


class WorkflowConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "wbcore.contrib.workflow"

    def ready(self):
        autodiscover_modules("workflows")
        from wbcore.contrib.workflow.dispatch import check_workflow_for_instance
        from wbcore.contrib.workflow.serializers.signals import (
            add_workflow_next_buttons_to_instance,
        )
        from wbcore.contrib.workflow.sites import workflow_site

        for model_class in (
            workflow_site.registered_model_classes_serializer_map.keys()
        ):  # we don't use items because the map is a custom cachedictionary where
            serializer_class = workflow_site.registered_model_classes_serializer_map[model_class]
            post_save.connect(check_workflow_for_instance, sender=model_class, weak=False)
            add_dynamic_button.connect(add_workflow_next_buttons_to_instance, sender=serializer_class)



from django.core.exceptions import ValidationError
from django.db.models import DecimalField, Field, FloatField, PositiveIntegerField


class AbstractDynamicField(Field):
    dependencies = []

    def __init__(self, *args, dependencies: list | None = None, **kwargs):
        blank = kwargs.pop("blank", True)
        null = kwargs.pop("null", True)
        self.dependencies = dependencies if dependencies else []
        super().__init__(*args, blank=blank, null=null, **kwargs)


class DynamicDecimalField(DecimalField, AbstractDynamicField):
    description = "Use a custom callback to compute this field if None"


class DynamicFloatField(FloatField, AbstractDynamicField):
    description = "Use a custom callback to compute this field if None"


def validate_year(year: int | None):
    if year and not len(str(year)) == 4:
        raise ValidationError("Please provide a valid 4-digit year.")


class YearField(PositiveIntegerField):
    default_validators = [validate_year]



from typing import Iterable

import django_filters
from django.core.exceptions import ObjectDoesNotExist
from django.forms.widgets import SelectMultiple, TextInput
from django.utils.http import urlencode
from django_filters.fields import ModelChoiceField, ModelMultipleChoiceField
from rest_framework.exceptions import ParseError
from rest_framework.reverse import reverse

from wbcore.filters.mixins import WBCoreFilterMixin


class ModelChoiceFilterMixin(WBCoreFilterMixin):
    MULTIPLE: bool = False

    def _validate_initial_with_request(self, initial, request, name):
        if request_default := request.GET.get(name):
            return request_default.split(",")
        return initial

    @classmethod
    def get_parsed_values(cls, queryset, value_ids: int | str | Iterable[int]):
        if isinstance(value_ids, str):
            value_ids = value_ids.split(",")
        elif isinstance(value_ids, int):
            value_ids = [value_ids]

        if isinstance(value_ids, list):
            for value_id in value_ids:
                try:
                    yield {
                        "value": value_id,
                        "label": str(queryset.get(id=value_id)),
                    }
                except ObjectDoesNotExist as e:
                    raise ParseError("Filter value invalid") from e

    def get_representation(self, request, name, view):
        representation, lookup_expr = super().get_representation(request, name, view)
        lookup_expr["input_properties"]["multiple"] = self.MULTIPLE

        queryset = self.get_queryset(request)

        if hasattr(queryset.model, "get_label_key"):
            label_key = queryset.model.get_label_key()
        else:
            label_key = self.label_key

        url = reverse(self.endpoint, request=request)
        if self.filter_params:
            if callable(self.filter_params):
                filter_params = self.filter_params(request, view)
            else:
                filter_params = self.filter_params
            # we need to convert any list into comma seperated string
            for key, value in filter_params.items():
                if isinstance(value, list):
                    filter_params[key] = ",".join(map(lambda x: str(x), value))

            url += f"?{urlencode(filter_params, doseq=True)}"
        lookup_expr["input_properties"]["endpoint"] = {
            "url": url,
            "value_key": self.value_key,
            "label_key": label_key,
        }

        if initial_ids := lookup_expr["input_properties"].get("initial", None):
            values = self.get_parsed_values(queryset, initial_ids)
            lookup_expr["input_properties"]["initial"] = list(values) if self.MULTIPLE else next(values)

        return representation, lookup_expr


class ModelMultipleChoiceFilter(ModelChoiceFilterMixin, django_filters.ModelMultipleChoiceFilter):
    class SimpleModelMultipleChoiceField(ModelMultipleChoiceField):
        """
        field class that define a simple text input as widget (instead of select). This is necessary in order to use the
        browsable api for model with a lot of items. Without it, the widget would load all the queryset option and will probably destroy the performance
        """

        class SimpleSelectMultiple(SelectMultiple):
            template_name = "django/forms/widgets/text.html"

        widget = SimpleSelectMultiple

    MULTIPLE: bool = True
    field_class = SimpleModelMultipleChoiceField
    filter_type = "select"

    def __init__(self, *args, **kwargs):
        self.endpoint = kwargs.pop("endpoint", None)
        self.filter_params = kwargs.pop("filter_params", None)
        self.value_key = kwargs.pop("value_key", None)
        self.label_key = kwargs.pop("label_key", None)
        # TODO: This is monkeypatched. Make sure that the CSVWidget is set here and only here!
        if "widget" not in kwargs:
            kwargs["widget"] = django_filters.widgets.CSVWidget
        super().__init__(*args, **kwargs)

        # django filter sets it to True by default. In our case, the fitlering will happen on primary keys, so we do not expect any duplicate. Furthermore, for table without explicit unique constraint, using "distinct" leads to unexpected results (i.e. row with same value are dropped)
        self.distinct = False


class ModelChoiceFilter(ModelChoiceFilterMixin, django_filters.ModelChoiceFilter):
    class SimpleModelChoiceField(ModelChoiceField):
        """
        field class that define a simple text input as widget (instead of select). This is necessary in order to use the
        browsable api for model with a lot of items. Without it, the widget would load all the queryset option and will probably destroy the performance
        """

        widget = TextInput

    field_class = SimpleModelChoiceField
    filter_type = "select"
    MULTIPLE: bool = False

    def __init__(self, *args, **kwargs):
        self.endpoint = kwargs.pop("endpoint", None)
        self.value_key = kwargs.pop("value_key", None)
        self.filter_params = kwargs.pop("filter_params", None)
        self.label_key = kwargs.pop("label_key", None)
        super().__init__(*args, **kwargs)



from django.db import connection


def app_pre_migration(sender, app_config, **kwargs):
    cur = connection.cursor()
    cur.execute("CREATE EXTENSION IF NOT EXISTS pg_trgm;")
    cur.execute("CREATE EXTENSION IF NOT EXISTS btree_gin;")



import pytest
from django.core.exceptions import ValidationError
from wbcore.contrib.color.fields import validate_color


class TestColorield:
    @pytest.mark.parametrize(
        ("input", "valid"),
        [
            ("#1234567", False),
            ("259868", False),
            ("#25986", False),
            ("", False),
            ("#d225b0", True),
            ("#4839a3", True),
            ("#259868", True),
            (None, True),
        ],
    )
    def test_color_validator(self, input, valid):
        if valid:
            assert input == validate_color(input)
        else:
            with pytest.raises(ValidationError):
                validate_color(input)



# Generated by Django 4.1.9 on 2023-05-24 08:39

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0002_notificationusertoken_unique_user_token_device"),
    ]

    operations = [
        migrations.AddField(
            model_name="notificationusertoken",
            name="updated",
            field=models.DateTimeField(auto_now=True),
        ),
    ]



import factory

from .models import Tag, TagGroup


class TagFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Tag

    title = factory.Faker("name")
    description = factory.Faker("paragraph")

    @factory.post_generation
    def groups(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            for group in extracted:
                self.groups.add(group)


class TagGroupFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = TagGroup

    title = factory.Faker("name")



from wbcore.metadata.configs.display.instance_display.shortcuts import (
    Display,
    create_simple_display,
)
from wbcore.metadata.configs.preview import PreviewViewConfig


class DocumentPreviewConfig(PreviewViewConfig):
    def get_display(self) -> Display:
        return create_simple_display([["name", "document_type", "description"]])



# Generated by Django 5.0.13 on 2025-03-13 12:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0007_notificationtype_resource_button_label'),
    ]

    operations = [
        migrations.AddField(
            model_name='notificationtype',
            name='is_lock',
            field=models.BooleanField(default=False),
        ),
    ]



import django_filters
from django.utils.translation import gettext_lazy as _

from wbcore import filters as wb_filters
from wbcore.contrib.currency.models import Currency
from wbcore.contrib.geography.models import Geography

from ..models import (
    AddressContact,
    BankingContact,
    ContactLocationChoices,
    ContactTelephoneChoices,
    EmailContact,
    Entry,
    SocialMediaContact,
    TelephoneContact,
    WebsiteContact,
)


class ContactPersonFilter(wb_filters.FilterSet):
    company_contact = wb_filters.BooleanFilter(
        label=_("Company Contact"),
        method="boolean_is_company_contact",
    )

    def boolean_is_company_contact(self, queryset, name, value):
        if value is None:
            return queryset
        return queryset.filter(company_contact=value)


class EmailContactCompanyFilter(wb_filters.FilterSet):
    location = wb_filters.MultipleChoiceFilter(
        label=_("Locations"), choices=ContactLocationChoices.choices, widget=django_filters.widgets.CSVWidget
    )

    class Meta:
        model = EmailContact
        fields = {
            "primary": ["exact"],
            "address": ["exact", "icontains"],
        }


class EmailContactPersonFilter(EmailContactCompanyFilter, ContactPersonFilter):
    pass


class AddressContactCompanyFilter(wb_filters.FilterSet):
    geography_city = wb_filters.ModelChoiceFilter(
        label=_("City"),
        queryset=Geography.cities.all(),
        endpoint=Geography.get_representation_endpoint(),
        value_key=Geography.get_representation_value_key(),
        label_key=Geography.get_representation_label_key(),
        method="filter_city",
        filter_params={"level": 3},
    )

    geography_country = wb_filters.ModelChoiceFilter(
        label=_("Country"),
        queryset=Geography.countries.all(),
        endpoint=Geography.get_representation_endpoint(),
        value_key=Geography.get_representation_value_key(),
        label_key=Geography.get_representation_label_key(),
        method="filter_country",
        filter_params={"level": 1},
    )

    geography_state = wb_filters.ModelChoiceFilter(
        label=_("State"),
        queryset=Geography.states.all(),
        endpoint=Geography.get_representation_endpoint(),
        value_key=Geography.get_representation_value_key(),
        label_key=Geography.get_representation_label_key(),
        method="filter_state",
        filter_params={"level": 2},
    )

    geography_continent = wb_filters.ModelChoiceFilter(
        label=_("Continent"),
        queryset=Geography.continents.all(),
        endpoint=Geography.get_representation_endpoint(),
        value_key=Geography.get_representation_value_key(),
        label_key=Geography.get_representation_label_key(),
        method="filter_continent",
        filter_params={"level": 0},
    )

    location = wb_filters.MultipleChoiceFilter(
        label=_("Locations"), choices=ContactLocationChoices.choices, widget=django_filters.widgets.CSVWidget
    )

    def filter_city(self, queryset, name, value):
        if value:
            return queryset.filter(geography_city=value)
        return queryset

    def filter_country(self, queryset, name, value):
        if value:
            return queryset.filter(geography_city__parent__parent=value)
        return queryset

    def filter_state(self, queryset, name, value):
        if value:
            return queryset.filter(geography_city__parent=value)
        return queryset

    def filter_continent(self, queryset, name, value):
        if value:
            return queryset.filter(geography_city__parent__parent__parent=value)
        return queryset

    class Meta:
        model = AddressContact
        fields = {
            "street": ["exact", "icontains"],
            "street_additional": ["exact", "icontains"],
            "zip": ["exact", "icontains"],
            "primary": ["exact"],
        }


class AddressContactPersonFilter(AddressContactCompanyFilter, ContactPersonFilter):
    pass


class TelephoneContactCompanyFilter(wb_filters.FilterSet):
    location = wb_filters.MultipleChoiceFilter(
        label=_("Locations"), choices=ContactLocationChoices.choices, widget=django_filters.widgets.CSVWidget
    )
    telephone_type = wb_filters.MultipleChoiceFilter(
        label=_("Types"), choices=ContactTelephoneChoices.choices, widget=django_filters.widgets.CSVWidget
    )

    class Meta:
        model = TelephoneContact
        fields = {
            "number": ["exact", "icontains"],
            "primary": ["exact"],
        }


class TelephoneContactPersonFilter(TelephoneContactCompanyFilter, ContactPersonFilter):
    pass


class WebsiteContactCompanyFilter(wb_filters.FilterSet):
    location = wb_filters.MultipleChoiceFilter(
        label=_("Locations"), choices=ContactLocationChoices.choices, widget=django_filters.widgets.CSVWidget
    )

    class Meta:
        model = WebsiteContact
        fields = {
            "url": ["exact", "icontains"],
            "primary": ["exact"],
        }


class WebsiteContactPersonFilter(WebsiteContactCompanyFilter, ContactPersonFilter):
    pass


class BankingContactCompanyFilter(wb_filters.FilterSet):
    currency = wb_filters.ModelMultipleChoiceFilter(
        label=_("Currencies"),
        queryset=Currency.objects.all(),
        endpoint=Currency.get_representation_endpoint(),
        value_key=Currency.get_representation_value_key(),
        label_key=Currency.get_representation_label_key(),
    )
    location = wb_filters.MultipleChoiceFilter(
        label=_("Locations"), choices=ContactLocationChoices.choices, widget=django_filters.widgets.CSVWidget
    )
    edited__lte = wb_filters.DateTimeFilter(
        label=_("Edited"),
        lookup_expr="lte",
        field_name="edited",
    )

    edited__gte = wb_filters.DateTimeFilter(
        label=_("Edited"),
        lookup_expr="gte",
        field_name="edited",
    )

    class Meta:
        model = BankingContact
        fields = {
            "status": ["exact"],
            "institute": ["exact", "icontains"],
            "institute_additional": ["exact", "icontains"],
            "iban": ["exact", "icontains"],
            "swift_bic": ["exact", "icontains"],
            "primary": ["exact"],
            "additional_information": ["exact", "icontains"],
        }


class BankingContactPersonFilter(BankingContactCompanyFilter, ContactPersonFilter):
    pass


class BankingContactFilter(BankingContactCompanyFilter):
    entry = wb_filters.ModelMultipleChoiceFilter(
        label=_("Entries"),
        queryset=Entry.objects.all(),
        endpoint=Entry.get_representation_endpoint(),
        value_key=Entry.get_representation_value_key(),
        label_key=Entry.get_representation_label_key(),
    )


class SocialMediaContactCompanyFilter(wb_filters.FilterSet):
    location = wb_filters.MultipleChoiceFilter(
        label=_("Locations"),
        choices=ContactLocationChoices.choices,
        widget=django_filters.widgets.CSVWidget,
    )
    platform = wb_filters.MultipleChoiceFilter(
        label=_("Social Media Platform"),
        choices=SocialMediaContact.Platforms.choices,
        widget=django_filters.widgets.CSVWidget,
    )

    class Meta:
        model = SocialMediaContact
        fields = {"url": ["exact", "icontains"], "primary": ["exact"]}


class SocialMediaContactPersonFilter(SocialMediaContactCompanyFilter, ContactPersonFilter):
    pass



import pytest

from wbcore.cache.decorators import cache_table
from wbcore.cache.registry import periodic_cache_registry


@pytest.mark.parametrize(
    "timeout, key_prefix, periodic_caching_view_kwargs, periodic_caching_get_parameters",
    [(2, "Foo", [{"a": "a"}], [{"b": "b"}])],
)
def test_cache_table(timeout, key_prefix, periodic_caching_view_kwargs, periodic_caching_get_parameters):
    @cache_table(
        timeout=timeout,
        key_prefix=key_prefix,
        periodic_caching=True,
        periodic_caching_view_kwargs=periodic_caching_view_kwargs,
        periodic_caching_get_parameters=periodic_caching_get_parameters,
    )
    class CacheTable:
        pass

    assert CacheTable.CACHE_ENABLED is True
    assert CacheTable.CACHE_TIMEOUT == timeout
    assert CacheTable.CACHE_KEY_PREFIX == key_prefix
    cache_entry = periodic_cache_registry.classes[0]
    assert cache_entry.view_class == CacheTable
    assert cache_entry.view_kwargs == periodic_caching_view_kwargs
    assert cache_entry.get_parameters == periodic_caching_get_parameters



from django.dispatch import receiver
from django.urls import resolve
from rest_framework.request import Request
from rest_framework.reverse import reverse

from wbcore import serializers
from wbcore.contrib.icons.icons import WBIcon
from wbcore.enums import RequestType
from wbcore.metadata.configs.buttons import ActionButton
from wbcore.metadata.configs.buttons.enums import ButtonDefaultColor
from wbcore.metadata.configs.display.instance_display.shortcuts import create_simple_display
from wbcore.signals.instance_buttons import add_extra_button


class AutoTranslateSerializer(serializers.Serializer):
    override_existing_data = serializers.BooleanField(default=False)


@receiver(add_extra_button)
def add_auto_translate_action_button(sender, instance, request: Request, view, pk=None, **kwargs):
    if instance and pk and view and hasattr(view, "auto_translate"):
        url = reverse(resolve(request.path).view_name.replace("detail", "auto-translate"), args=[pk], request=request)
        return ActionButton(
            method=RequestType.POST,
            icon=WBIcon.DEAL.icon,
            color=ButtonDefaultColor.SUCCESS,
            endpoint=url,
            label="Auto Translate",
            action_label="Auto Translate",
            description_fields="You will automatically translate all fields available for translation.",
            serializer=AutoTranslateSerializer,
            instance_display=create_simple_display([["override_existing_data"]]),
        )



import pytest
from django.forms.models import model_to_dict
from rest_framework.test import APITestCase
from wbcore.contrib.example_app.factories import StadiumFactory
from wbcore.contrib.example_app.serializers import StadiumModelSerializer


@pytest.mark.django_db
class TestStadiumModelSerializer(APITestCase):
    def test_stadium_serializer(self):
        stadium_data: dict = model_to_dict(StadiumFactory.build())
        stadium_serializer = StadiumModelSerializer(data=stadium_data)

        self.assertTrue(stadium_serializer.is_valid())



import factory
import pytz

from ..models import UserActivity
from .users import UserFactory


class UserActivityFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = UserActivity

    IP = factory.Faker("ipv4")
    date = factory.Faker("date_time", tzinfo=pytz.utc)
    user = factory.SubFactory(UserFactory)
    # status
    # jti
    # type
    user_agent_info = factory.Faker("sentences")
    latest_refresh = factory.Faker("date_time", tzinfo=pytz.utc)



# Generated by Django 5.0.6 on 2024-08-13 09:11
import django.contrib.postgres.fields.ranges
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("example_app", "0005_sportperson_profile_image"),
    ]

    operations = [
        migrations.CreateModel(
            name="Season",
            options={"verbose_name": "Season", "verbose_name_plural": "Seasons"},
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("date_range", django.contrib.postgres.fields.ranges.DateRangeField()),
                ("name", models.CharField(blank=True, max_length=255)),
                ("file", models.FileField(blank=True, null=True, upload_to="season_files/")),
                (
                    "league",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, related_name="seasons", to="example_app.league"
                    ),
                ),
                (
                    "top_scorer",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="top_scorer_seasons",
                        to="example_app.player",
                    ),
                ),
                (
                    "winner",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="won_seasons",
                        to="example_app.team",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="player",
            name="is_active",
            field=models.BooleanField(default=True, verbose_name="Is Active"),
        ),
        migrations.AddField(
            model_name="player",
            name="is_injured",
            field=models.BooleanField(default=False, verbose_name="Is Injured"),
        ),
        migrations.AddField(
            model_name="player",
            name="player_strength",
            field=models.IntegerField(
                default=3,
                validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(5)],
                verbose_name="Player Strength",
            ),
        ),
        migrations.AddField(
            model_name="player",
            name="game_activity",
            field=models.FloatField(
                default=0,
                validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)],
                verbose_name="Game Activity",
            ),
        ),
        migrations.AddField(
            model_name="stadium",
            name="guest_rating",
            field=models.IntegerField(
                default=3,
                validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(4)],
                verbose_name="Guest Rating",
            ),
        ),
        migrations.AddField(
            model_name="team",
            name="phone_number",
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name="Phone Number"),
        ),
        migrations.AddField(
            model_name="team",
            name="email",
            field=models.EmailField(blank=True, max_length=254, null=True, verbose_name="Email Address"),
        ),
        migrations.AddField(
            model_name="team",
            name="duration_since_last_win",
            field=models.DurationField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="event",
            name="event_description",
            field=models.TextField(blank=True, max_length=512, null=True, verbose_name="Event Description"),
        ),
        migrations.AddField(
            model_name="match",
            name="winner",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="won_matches",
                to="example_app.team",
            ),
        ),
    ]



from wbcore import serializers as wb_serializers
from wbcore.contrib.workflow.models import Display


class DisplayModelSerializer(wb_serializers.ModelSerializer):
    class Meta:
        model = Display
        fields = (
            "id",
            "name",
            "grid_template_areas",
            "_additional_resources",
        )


class DisplayRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbcore:workflow:display-detail")

    class Meta:
        model = Display
        fields = (
            "id",
            "name",
            "_detail",
        )



from django.db.models import Model
from django.utils.translation import gettext as _
from rest_framework.exceptions import ValidationError
from rest_framework.reverse import reverse

from wbcore import serializers as wb_serializers
from wbcore.content_type.serializers import ContentTypeRepresentationSerializer
from wbcore.contrib.icons import WBIcon
from wbcore.contrib.workflow.models import Workflow
from wbcore.contrib.workflow.sites import workflow_site
from wbcore.enums import RequestType
from wbcore.metadata.configs import buttons as bt


class WorkflowModelSerializer(wb_serializers.ModelSerializer):
    _model = ContentTypeRepresentationSerializer(source="model", allowed_types=workflow_site.model_content_types)
    graph = wb_serializers.ImageField(required=False, read_only=True, label=_("Graph"))

    @wb_serializers.register_resource()
    def steps_inline(self, instance: Workflow, request, user) -> dict[str, str]:
        if not self.context.get("request"):
            return {}
        return {
            "steps": reverse("wbcore:workflow:step-workflow-list", args=[instance.pk], request=self.context["request"])
        }

    @wb_serializers.register_resource()
    def transitions_inline(self, instance: Workflow, request, user) -> dict[str, str]:
        if not self.context.get("request"):
            return {}
        return {
            "transitions": reverse(
                "wbcore:workflow:transition-workflow-list", args=[instance.pk], request=self.context["request"]
            )
        }

    @wb_serializers.register_resource()
    def processes_inline(self, instance: Workflow, request, user) -> dict[str, str]:
        if not self.context.get("request"):
            return {}
        return {
            "processes": reverse(
                "wbcore:workflow:process-workflow-list", args=[instance.pk], request=self.context["request"]
            )
        }

    @wb_serializers.register_resource()
    def data_inline(self, instance: Workflow, request, user) -> dict[str, str]:
        if not self.context.get("request"):
            return {}
        return {
            "data": reverse("wbcore:workflow:data-workflow-list", args=[instance.pk], request=self.context["request"])
        }

    @wb_serializers.register_dynamic_button()
    def start_workflow_buttons(self, instance: Workflow, request, user):
        buttons = []
        for step in instance.get_start_steps_for_workflow():
            buttons.append(
                bt.ActionButton(
                    method=RequestType.PATCH,
                    identifiers=("workflow:process",),
                    endpoint=f"{reverse('wbcore:workflow:workflow-start', args=[instance.pk], request=request)}?step_id={step.pk}",
                    label=_("Start {}").format(step.workflow.name),
                    icon=WBIcon.START.icon,
                    description_fields=_("Are you sure you want to start workflow {}?").format(step.workflow.name),
                    title=_("Start {}").format(step.workflow.name),
                    action_label=_("Starting {}").format(step.workflow.name),
                )
            )
        return buttons

    def validate(self, data: dict) -> dict:
        if "status_field" in data:
            status_field: str | None = data["status_field"]
        else:
            status_field: str | None = self.instance.status_field if self.instance else None
        if "model" in data:
            model: Model | None = data["model"]
        else:
            model: Model | None = self.instance.model if self.instance else None
        preserve_instance: bool | None = data.get(
            "preserve_instance", self.instance.preserve_instance if self.instance else None
        )

        if preserve_instance and not bool(model):
            raise ValidationError({"preserve_instance": _("Can only preserve the instance when a model is attached.")})

        if model and status_field:
            if not (model_class := model.model_class()):
                raise ValidationError({"model": _("Model not found.")})
            if not hasattr(model_class, status_field):
                raise ValidationError({"status_field": _("This model does not implement the specified status field.")})
        return data

    class Meta:
        dependency_map = {
            "status_field": ["model"],
            "preserve_instance": ["model"],
        }
        model = Workflow
        fields = (
            "id",
            "name",
            "single_instance_execution",
            "model",
            "_model",
            "status_field",
            "_additional_resources",
            "_buttons",
            "preserve_instance",
            "graph",
        )


class WorkflowRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbcore:workflow:workflow-detail")

    class Meta:
        model = Workflow
        fields = (
            "id",
            "name",
            "_detail",
        )



import pytest
from django.test import Client, TestCase
from django.urls import reverse
from wbcore.contrib.authentication.factories import SuperUserFactory
from wbcore.contrib.example_app.factories import MatchFactory, TeamFactory
from wbcore.contrib.example_app.models import Match
from wbcore.contrib.example_app.serializers import MatchModelSerializer
from wbcore.contrib.example_app.tests.test_viewsets.test_utils_viewsets import (
    get_create_view,
    get_delete_view,
    get_detail_view,
    get_partial_view,
    get_update_view,
)
from wbcore.contrib.example_app.viewsets import MatchModelViewSet


@pytest.mark.django_db
class TestMatchModelViewSet(TestCase):
    def setUp(self):
        self.user = SuperUserFactory()
        self.client = Client()
        self.client.force_login(user=self.user)
        self.list_url = reverse("example_app:match-list")
        self.detail_url_str = "example_app:match-detail"

    def test_list_view(self):
        response = self.client.get(self.list_url)
        self.assertEqual(response.status_code, 200)

    def test_create_view(self):
        match = MatchFactory()
        response = get_create_view(self.client, match, self.user, self.list_url, MatchModelViewSet)
        self.assertEqual(response.status_code, 201)
        self.assertTrue(Match.objects.filter(home=match.home, away=match.away).exists())

    def test_detail_view(self):
        instance = MatchFactory()
        response = get_detail_view(self.client, instance.pk, self.detail_url_str)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data["instance"]["id"], instance.id)

    def test_update_view(self):
        instance = MatchFactory(status=Match.MatchStatus.SCHEDULED)
        new_home_team = TeamFactory()
        instance.home = new_home_team
        response = get_update_view(self.client, instance, MatchModelSerializer, self.detail_url_str)
        instance.refresh_from_db()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data["instance"]["home"], instance.home.id)

    def test_partial_update_view(self):
        instance = MatchFactory()
        new_home_team = TeamFactory()
        response = get_partial_view(self.client, instance.id, {"home": new_home_team.id}, self.detail_url_str)
        instance.refresh_from_db()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data["instance"]["home"], instance.home.id)

    def test_delete_view(self):
        instance = MatchFactory()
        self.assertTrue(Match.objects.filter(pk=instance.pk).exists())
        response = get_delete_view(self.client, self.detail_url_str, instance.pk)
        self.assertEqual(response.status_code, 204)
        self.assertFalse(Match.objects.filter(pk=instance.pk).exists())



from django.utils.translation import gettext_lazy
from rest_framework.reverse import reverse

from wbcore.utils.importlib import import_from_dotted_path


class ConfigurationRegistry:
    DEFAULT_COMPANY_MODEL_VIEWSET = "wbcore.contrib.directory.viewsets.entries.CompanyModelViewSet"
    DEFAULT_COMPANY_REPRESENTATION_VIEWSET = "wbcore.contrib.directory.viewsets.entries.CompanyRepresentationViewSet"
    DEFAULT_COMPANY_MODEL_SERIALIZER = "wbcore.contrib.directory.serializers.companies.CompanyModelSerializer"
    DEFAULT_COMPANY_REPRESENTATION_SERIALIZER = (
        "wbcore.contrib.directory.serializers.entries.CompanyRepresentationSerializer"
    )
    DEFAULT_PERSON_MODEL_VIEWSET = "wbcore.contrib.directory.viewsets.entries.PersonModelViewSet"
    DEFAULT_PERSON_REPRESENTATION_VIEWSET = "wbcore.contrib.directory.viewsets.entries.PersonRepresentationViewSet"
    DEFAULT_PERSON_MODEL_SERIALIZER = "wbcore.contrib.directory.serializers.persons.PersonModelSerializer"
    DEFAULT_PERSON_REPRESENTATION_SERIALIZER = (
        "wbcore.contrib.directory.serializers.entries.PersonRepresentationSerializer"
    )

    @property
    def company_model_viewset(self):
        return import_from_dotted_path(self.DEFAULT_COMPANY_MODEL_VIEWSET)

    @property
    def company_representation_viewset(self):
        return import_from_dotted_path(self.DEFAULT_COMPANY_REPRESENTATION_VIEWSET)

    @property
    def company_model_serializer(self):
        return import_from_dotted_path(self.DEFAULT_COMPANY_MODEL_SERIALIZER)

    @property
    def company_representation_serializer(self):
        return import_from_dotted_path(self.DEFAULT_COMPANY_REPRESENTATION_SERIALIZER)

    @property
    def person_model_viewset(self):
        return import_from_dotted_path(self.DEFAULT_PERSON_MODEL_VIEWSET)

    @property
    def person_representation_viewset(self):
        return import_from_dotted_path(self.DEFAULT_PERSON_REPRESENTATION_VIEWSET)

    @property
    def person_model_serializer(self):
        return import_from_dotted_path(self.DEFAULT_PERSON_MODEL_SERIALIZER)

    @property
    def person_representation_serializer(self):
        return import_from_dotted_path(self.DEFAULT_PERSON_REPRESENTATION_SERIALIZER)


class DirectoryConfigurationMixin:
    WBCORE_PROFILE = "wbcore.contrib.directory.configurations.resolve_profile"

    DEFAULT_TIERING_HELP_TEXT = gettext_lazy(
        "The tier of the company with 1 being the top tier and 5 being the lowest tier."
    )


def resolve_profile(request):
    user = request.user
    return {
        "image": user.profile.profile_image.url if user.profile.profile_image else None,
        "endpoint": reverse("wbcore:authentication:userprofile-detail", args=[user.id], request=request),
    }


configuration_registry = ConfigurationRegistry()



from dataclasses import dataclass
from typing import Iterable

from wbcore.utils.importlib import parse_signal_received_for_module

from .enums import NavigationType
from .pages import Page, SerializedPage
from .signals import add_display_pages

type SerializedDisplay = dict[str, NavigationType | SerializedPage]


@dataclass
class Display:
    """A display contains multiple pages and a navigation type. When only 1 page is specified, the navigation type is irrelevant"""

    pages: Iterable[Page]
    navigation_type: NavigationType = NavigationType.TAB

    def serialize(
        self, view_config=None, view=None, request=None, key_prefix=None, parent_page=None
    ) -> SerializedDisplay:
        """Serializes a `Display`

        Returns:
            A dictionary with all pages and the navigation type

        """
        serialized_pages = [
            page.serialize(key_prefix=key_prefix, view_config=view_config, view=view, request=request)
            for page in self.pages
        ]
        # we only allow remote page registration if the display is the top display. We could argue that we want to allow dev to register pages for any nested display by passing down the page argument. This involves refactoring of the signal receivers
        if view_config and parent_page is None:
            for prefix, remote_pages in parse_signal_received_for_module(
                add_display_pages.send(view_config.__class__, request=request, view=view)
            ):
                for remote_page in remote_pages:
                    serialized_pages.append(remote_page.serialize(prefix, view_config, view, request))
        return {"navigationType": self.navigation_type.value, "pages": serialized_pages}



from unittest.mock import patch

import pytest
from wbcore.contrib.workflow.factories import UserStepFactory
from wbcore.contrib.workflow.models import Condition


@pytest.mark.django_db
class TestTransition:
    def test_all_conditions_satisfied_no_conditions(self, transition_factory, process_step_factory):
        transition = transition_factory()
        process_step = process_step_factory()
        assert transition.all_conditions_satisfied(process_step)

    @patch("wbcore.contrib.workflow.models.step.Step.set_failed")
    @patch("wbcore.contrib.workflow.models.process.ProcessStep.get_instance")
    def test_all_conditions_satisfied_no_instance(
        self,
        mock_get_instance,
        mock_failed,
        random_child_step_factory,
        transition_factory,
        condition_factory,
        process_step_factory,
    ):
        transition = transition_factory()
        condition_factory(transition=transition)
        step = random_child_step_factory(exclude_factories=[UserStepFactory])
        process_step = process_step_factory(step=step)
        mock_get_instance.return_value = None
        assert not transition.all_conditions_satisfied(process_step)
        assert mock_failed.call_args.args[0] == process_step

    @patch("wbcore.contrib.workflow.models.process.ProcessStep.get_instance")
    @patch("wbcore.contrib.workflow.models.condition.Condition.satisfied")
    def test_all_conditions_satisfied(
        self, mock_satisfied, mock_get_instance, transition_factory, condition_factory, process_step_factory
    ):
        transition = transition_factory()
        process_step = process_step_factory()
        attached_instance = process_step.process.instance
        condition_factory()
        condition_factory(transition=transition)
        condition_factory(transition=transition)
        mock_get_instance.return_value = attached_instance
        mock_satisfied.return_value = True
        assert transition.all_conditions_satisfied(process_step)
        assert mock_satisfied.call_args.args == (attached_instance,)
        assert mock_satisfied.call_count == 2

    @patch("wbcore.contrib.workflow.models.step.Step.set_failed")
    @patch("wbcore.contrib.workflow.models.process.ProcessStep.get_instance")
    def test_all_conditions_satisfied_errors(
        self,
        mock_get_instance,
        mock_failed,
        random_child_step_factory,
        transition_factory,
        condition_factory,
        process_step_factory,
    ):
        transition = transition_factory()
        step = random_child_step_factory(exclude_factories=[UserStepFactory])
        process_step = process_step_factory(step=step)
        attached_instance = process_step.process.instance
        condition_factory()
        condition_factory(
            transition=transition,
            operator=Condition.Operator.EQ,
            expected_value=attached_instance.first_name,
            negate_operator=False,
        )
        condition_factory(
            transition=transition,
            attribute_name="last_name",
            operator=Condition.Operator.EQ,
            expected_value=attached_instance.last_name,
            negate_operator=False,
        )
        condition_factory(
            transition=transition,
            attribute_name="Test",
            operator=Condition.Operator.EQ,
            expected_value=attached_instance.first_name,
            negate_operator=True,
        )
        mock_get_instance.return_value = attached_instance
        assert not transition.all_conditions_satisfied(process_step)
        assert mock_failed.call_args.args[0] == process_step

    @patch("wbcore.contrib.workflow.models.step.Step.set_failed")
    @patch("wbcore.contrib.workflow.models.process.ProcessStep.get_instance")
    def test_all_conditions_satisfied_false(
        self,
        mock_get_instance,
        mock_failed,
        random_child_step_factory,
        transition_factory,
        condition_factory,
        process_step_factory,
    ):
        transition = transition_factory()
        step = random_child_step_factory(exclude_factories=[UserStepFactory])
        process_step = process_step_factory(step=step)
        attached_instance = process_step.process.instance
        condition_factory()
        condition_factory(
            transition=transition,
            operator=Condition.Operator.EQ,
            expected_value=attached_instance.first_name,
            negate_operator=False,
        )
        condition_factory(
            transition=transition,
            operator=Condition.Operator.EQ,
            expected_value=attached_instance.first_name,
            negate_operator=True,
        )
        condition_factory(
            transition=transition,
            attribute_name="last_name",
            operator=Condition.Operator.EQ,
            expected_value=attached_instance.last_name,
            negate_operator=False,
        )
        mock_get_instance.return_value = attached_instance
        assert not transition.all_conditions_satisfied(process_step)
        assert not mock_failed.called



import pytest
from django.forms.models import model_to_dict
from rest_framework import status
from rest_framework.test import APIRequestFactory

from wbcore.contrib.authentication.factories import SuperUserFactory
from wbcore.contrib.tags.factories import (
    TagFactory,
    TagGroupFactory,
)
from wbcore.contrib.tags.viewsets import (
    TagGroupModelViewSet,
    TagGroupRepresentationViewSet,
    TagModelViewSet,
    TagRepresentationViewSet,
)


@pytest.mark.viewset_tests
@pytest.mark.with_db
@pytest.mark.django_db
class TestTagViewSets:
    @pytest.fixture()
    def api_rf(self):
        factory = APIRequestFactory()
        return factory

    @pytest.fixture()
    def super_user(self):
        super_user = SuperUserFactory()
        return super_user

    @pytest.mark.parametrize(
        "viewset, factory",
        [
            (TagGroupModelViewSet, TagGroupFactory),
            (TagModelViewSet, TagFactory),
            (TagGroupRepresentationViewSet, TagGroupFactory),
            (TagRepresentationViewSet, TagFactory),
        ],
    )
    def test_get_list(self, viewset, factory, api_rf, super_user):
        # Arrange
        request = api_rf.get("")
        request.user = super_user
        factory.create_batch(3)
        vs = viewset.as_view({"get": "list"})
        # Act
        response = vs(request)
        # Assert
        assert response.status_code == status.HTTP_200_OK
        assert len(response.data["results"]) == 3
        assert not response.data.get("instance")

    @pytest.mark.parametrize(
        "viewset, factory",
        [
            (TagGroupModelViewSet, TagGroupFactory),
            (TagModelViewSet, TagFactory),
            (TagGroupRepresentationViewSet, TagGroupFactory),
            (TagRepresentationViewSet, TagFactory),
        ],
    )
    def test_get_instance(self, viewset, factory, api_rf, super_user):
        # Arrange
        request = api_rf.get("")
        request.user = super_user
        instance = factory()
        vs = viewset.as_view({"get": "retrieve"})
        # Act
        response = vs(request, pk=instance.pk)
        # Assert
        assert response.status_code == status.HTTP_200_OK
        assert response.data["instance"]["id"] == instance.id
        assert not response.data.get("results")

    @pytest.mark.parametrize(
        "viewset, factory",
        [
            (TagGroupModelViewSet, TagGroupFactory),
            (TagModelViewSet, TagFactory),
        ],
    )
    def test_delete_instance(self, viewset, factory, api_rf, super_user):
        # Arrange
        request = api_rf.delete("")
        request.user = super_user
        instance = factory()
        vs = viewset.as_view({"delete": "destroy"})
        model = viewset.get_model()
        # Act
        response = vs(request, pk=instance.pk)
        # Assert
        assert response.status_code == status.HTTP_204_NO_CONTENT
        assert not model.objects.filter(id=instance.id).exists()

    @pytest.mark.parametrize(
        "viewset, factory",
        [
            (TagGroupModelViewSet, TagGroupFactory),
            (TagModelViewSet, TagFactory),
        ],
    )
    def test_update_instance(self, viewset, factory, api_rf, super_user):
        # Arrange
        instance = factory()
        new_data = model_to_dict(factory.build())
        new_title = instance.title + "Foo Bar"
        new_data["id"] = instance.id
        new_data["title"] = new_title
        request = api_rf.put("", data=new_data, format="json")
        request.user = super_user
        vs = viewset.as_view({"put": "update"})
        # Act
        response = vs(request, pk=instance.pk)
        updated_instance = response.data.get("instance")
        print("RESPONSE:----------------->\n", response.data)  # noqa
        # Assert
        assert response.status_code == status.HTTP_200_OK
        assert updated_instance["id"] == instance.id
        assert updated_instance["title"] == new_title
        assert updated_instance["title"] != instance.title

    @pytest.mark.parametrize(
        "viewset, factory",
        [
            (TagGroupModelViewSet, TagGroupFactory),
            (TagModelViewSet, TagFactory),
        ],
    )
    def test_patch_instance(self, viewset, factory, api_rf, super_user):
        # Arrange
        instance = factory()
        new_title = instance.title + "Foo Bar"
        request = api_rf.patch("", data={"title": new_title})
        request.user = super_user
        vs = viewset.as_view({"patch": "partial_update"})
        # Act
        response = vs(request, pk=instance.pk)
        updated_instance = response.data.get("instance")
        # Assert
        assert response.status_code == status.HTTP_200_OK
        assert updated_instance["title"] == new_title
        assert updated_instance["title"] != instance.title



from configurations import values


class Media:
    MEDIA_URL = "/media/"


class LocalMedia(Media):
    @property
    def MEDIA_ROOT(self):  # noqa
        return self.BASE_DIR.joinpath("mediafiles")


class S3Media(Media):
    AWS_ACCESS_KEY_ID = values.Value("minio_access_key", environ_prefix=None)
    AWS_SECRET_ACCESS_KEY = values.Value("minio_secret_access_key", environ_prefix=None)

    AWS_S3_REGION_NAME = values.Value("", environ_prefix=None)
    AWS_S3_ENDPOINT_URL = values.Value("http://localhost:9000", environ_prefix=None)
    AWS_S3_SIGNATURE_VERSION = values.Value("s3v4", environ_prefix=None)
    AWS_DEFAULT_ACL = values.Value(None, environ_prefix=None)
    AWS_S3_FILE_OVERWRITE = values.BooleanValue(False, environ_prefix=None)



from django.utils.translation import gettext_lazy as _

from wbcore import filters
from wbcore.contrib.example_app.models import Role, SportPerson


class RoleFilter(filters.FilterSet):
    sport_persons = filters.ModelChoiceFilter(
        label=_("Associated Person"),
        queryset=SportPerson.objects.all(),
        endpoint=SportPerson.get_representation_endpoint(),
        value_key=SportPerson.get_representation_value_key(),
        label_key=SportPerson.get_representation_label_key(),
    )

    class Meta:
        model = Role
        fields = {"title": ("exact",)}



from django.urls import include, path

from wbcore.routers import WBCoreRouter

from . import viewsets

router = WBCoreRouter()
router.register(r"calendaritem", viewsets.InfiniteCalendarItemViewSet, basename="calendaritem")
router.register(
    r"calendaritemrepresentation", viewsets.CalendarItemRepresentationViewSet, basename="calendaritemrepresentation"
)
router.register(r"owncalendar", viewsets.OwnCalendarItemViewSet, basename="owncalendar")
router.register(
    r"conferenceroomrepresentation",
    viewsets.ConferenceRoomRepresentationViewSet,
    basename="conferenceroomrepresentation",
)
router.register(r"conferenceroom", viewsets.ConferenceRoomModelViewSet, basename="conferenceroom")
router.register(r"buildingrepresentation", viewsets.BuildingRepresentationViewSet, basename="buildingrepresentation")
router.register(r"building", viewsets.BuildingModelViewSet, basename="building")


urlpatterns = [
    path("", include(router.urls)),
    path("ics/", viewsets.get_ics, name="get_ics"),
]



import pytest
from pytest_mock import MockerFixture
from rest_framework.test import APIRequestFactory

from wbcore.contrib.authentication.models import Group, User
from wbcore.contrib.workflow.models import ProcessStep, Step, Transition, UserStep
from wbcore.contrib.workflow.serializers import (
    AssignedProcessStepSerializer,
    ProcessStepModelSerializer,
)
from wbcore.test.utils import get_or_create_superuser


@pytest.fixture
def process_step(mocker: MockerFixture):
    mocked_process_step = mocker.MagicMock(spec=ProcessStep)
    mocked_process_step.pk = 1
    mocked_process_step.state = ProcessStep.StepState.ACTIVE
    return mocked_process_step


class TestProcessStep:
    @pytest.fixture
    def transitions(self, mocker: MockerFixture):
        transition_1 = mocker.MagicMock(spec=Transition)
        transition_1.pk = 101
        transition_1.name = "Transition One"
        transition_1.icon = "icon_1"

        transition_2 = mocker.MagicMock(spec=Transition)
        transition_2.pk = 102
        transition_2.name = "Transition Two"
        transition_2.icon = "icon_2"

        return (transition_1, transition_2)

    @pytest.fixture
    def casted_step(self, mocker: MockerFixture):
        mocked_casted_step = mocker.MagicMock(spec=Step, assignee_method=False)
        return mocked_casted_step

    @pytest.fixture
    def user_step(self, mocker: MockerFixture, casted_step, transitions):
        mocked_user_step = mocker.MagicMock(spec=UserStep)
        mocked_user_step.step_type = Step.StepType.USERSTEP
        mocked_user_step.get_casted_step.return_value = casted_step
        mocked_user_step.get_all_valid_outgoing_transitions.return_value = [*transitions]
        return mocked_user_step

    @pytest.fixture(autouse=True)
    def mocked_patch_path(self, mocker: MockerFixture):
        mocker.patch(
            "wbcore.contrib.workflow.models.workflow",
            return_value="/dummy-url/",
        )
        mocker.patch(
            "wbcore.contrib.workflow.dispatch.ContentType.objects.get_for_model",
            return_value="content_type",
        )

    @pytest.mark.parametrize("user", ["super_user", "user", "group"])
    def test_next_process_step_buttons(self, mocker: MockerFixture, user, process_step, user_step):
        """
        This test verifies that the method next_process_step_buttons returns buttons when:
        - "super_user": The user is a superuser.
        - "user": The user is the assignee of the process step.
        - "group": The user belongs to the group assigned to the process step.
        """
        # Arrange
        mocked_process_step = process_step
        mocked_process_step.step = user_step

        request_user = mocker.MagicMock(spec=User, is_superuser=(user == "super_user"))
        if user == "user":
            mocked_process_step.assignee = request_user
        elif user == "group":
            mocked_group = mocker.MagicMock(spec=Group)
            mocked_group.user_set.all.return_value = [request_user]
            mocked_process_step.group = mocked_group
        request = mocker.MagicMock(user=request_user)

        # Act
        serializer = ProcessStepModelSerializer(mocked_process_step, context={"request": request})
        data = serializer.data
        button_labels = {button["label"] for button in data["_buttons"]}

        # Assert
        assert len(data["_buttons"]) == 2
        assert button_labels == {"Transition One", "Transition Two"}

    @pytest.mark.parametrize("reason", ["finished_state", "not_user_step"])
    def test_no_transition_buttons_when_step_is_not_eligible(
        self, mocker: MockerFixture, reason, process_step, user_step
    ):
        """
        Test that no transition buttons appear if:
        - The process step is in a non-active state, OR
        - The step is not a UserStep type.
        """
        # Arrange
        mocked_process_step = process_step
        if reason == "inactive_state":
            mocked_process_step.state = ProcessStep.StepState.FINISHED
        else:
            user_step.step_type = Step.StepType.SPLITSTEP

        super_user = mocker.MagicMock(spec=User, is_superuser=True)
        request = mocker.MagicMock(user=super_user)
        # Act
        serializer = ProcessStepModelSerializer(mocked_process_step, context={"request": request})
        data = serializer.data

        # Assert
        assert len(data["_buttons"]) == 0

    def test_next_process_step_buttons_not_assigned(self, mocker: MockerFixture, process_step, user_step, casted_step):
        """
        No buttons when there is a assignee_method
        """
        # Arrange
        request_user = mocker.MagicMock(spec=User, is_superuser=False)
        request = mocker.MagicMock(user=request_user)

        mocked_group = mocker.MagicMock(spec=Group)
        mocked_group.user_set.all.return_value = [request_user]

        mocked_casted_step = casted_step
        mocked_casted_step.assignee_method = True

        mocked_user_step = user_step
        mocked_user_step.get_casted_step.return_value = casted_step

        mocked_process_step = process_step
        mocked_process_step.step = mocked_user_step
        mocked_process_step.group = mocked_group

        # Act
        serializer = ProcessStepModelSerializer(mocked_process_step, context={"request": request})
        data = serializer.data
        # Assert
        assert len(data["_buttons"]) == 0

    def test_next_process_step_buttons_no_transitions(
        self, mocker: MockerFixture, process_step, user_step, casted_step
    ):
        """
        No transitions no buttons
        """
        # Arrange
        request_user = mocker.MagicMock(spec=User, is_superuser=True)
        request = mocker.MagicMock(user=request_user)

        mocked_user_step = user_step
        mocked_user_step.get_all_valid_outgoing_transitions.return_value = []

        mocked_process_step = process_step
        mocked_process_step.step = user_step

        # Act
        serializer = ProcessStepModelSerializer(mocked_process_step, context={"request": request})
        data = serializer.data
        # Assert
        assert len(data["_buttons"]) == 0


class TestAssignedProcessStep:
    @pytest.fixture
    def process_step_with_assignee(self, mocker: MockerFixture, process_step):
        mocked_step = mocker.MagicMock(spec=Step)
        mocked_step.id = 42
        mocked_user = mocker.MagicMock(spec=User)
        mocked_user.id = 3
        process_step.step = mocked_step
        process_step.assignee = mocked_user
        return process_step

    def test_get_instance_endpoint_no_request(self, mocker: MockerFixture, process_step_with_assignee):
        serializer = AssignedProcessStepSerializer(process_step_with_assignee)
        assert serializer.data
        assert serializer.data["instance_endpoint"] == ""

    @pytest.mark.django_db
    @pytest.mark.with_db
    @pytest.mark.parametrize("with_instance", [True, False])
    def test_get_instance_endpoint_no_attached_instance(self, process_step_factory, with_instance):
        api_factory = APIRequestFactory()
        process_step = (
            process_step_factory()
            if with_instance
            else process_step_factory(process__instance=None, process__instance_id=None)
        )
        request = api_factory.get("")
        request.user = get_or_create_superuser()
        serializer = AssignedProcessStepSerializer(process_step, context={"request": request})
        assert serializer.data
        if with_instance:
            assert "processstep" not in serializer.data["instance_endpoint"]
            assert "person" in serializer.data["instance_endpoint"]
        else:
            assert "processstep" in serializer.data["instance_endpoint"]



from contextlib import suppress
from typing import Any

from langchain_core.language_models import BaseChatModel
from langchain_core.messages import SystemMessage
from langchain_core.messages.base import BaseMessage
from langchain_core.output_parsers import PydanticOutputParser, StrOutputParser
from langchain_core.prompts import ChatPromptTemplate
from langchain_openai import ChatOpenAI
from pydantic import BaseModel, ValidationError

type Prompt = str | list[BaseMessage] | BaseMessage


def _convert_prompt_to_chat_prompt_template(prompt: Prompt, format_instructions=None) -> ChatPromptTemplate:
    if isinstance(prompt, BaseMessage):
        prompt = [prompt]
    if isinstance(prompt, str):
        prompt = SystemMessage(content=prompt)
    messages = []
    for index, msg in enumerate(prompt):
        content = msg.content
        if index == 0 and format_instructions:
            if "{format_instructions}" not in content:
                content += "\n{format_instructions}"
        messages.append((msg.type, content))
    return ChatPromptTemplate.from_messages(messages).partial(format_instructions=format_instructions)


def run_llm(
    prompt: Prompt,
    output_model: "type[BaseModel] | None" = None,
    chat_model: type[BaseChatModel] = ChatOpenAI,
    chat_model_name: str = "gpt-4o-mini",
    max_tokens: int = 16000,
    extra_tools: list[dict[str, str]] | None = None,
    query: dict[str, Any] | None = None,
    **llm_kwargs,
) -> tuple[BaseModel | None, BaseMessage]:
    llm = chat_model(model=chat_model_name, max_tokens=max_tokens, **llm_kwargs)  # type: ignore
    tools = []
    if not query:
        query = {}

    if extra_tools:
        tools.extend(extra_tools)
    if output_model:
        tools.append(output_model)
    if tools:
        llm = llm.bind_tools(tools)

    # Set up a parser
    if output_model:
        parser = PydanticOutputParser(pydantic_object=output_model)
    else:
        parser = StrOutputParser()
    try:
        format_instructions = parser.get_format_instructions()
    except NotImplementedError:
        format_instructions = None
    chat_prompt_template = _convert_prompt_to_chat_prompt_template(prompt, format_instructions=format_instructions)

    chain = chat_prompt_template | llm

    response = chain.invoke(query)  # type: ignore
    if output_model:
        with suppress(ValidationError, IndexError):
            return output_model.model_validate(response.tool_calls[0]["args"]), response
    return None, response



import factory

from wbcore.contrib.agenda.models import Building, ConferenceRoom


class BuildingFactory(factory.django.DjangoModelFactory):
    name = factory.Faker("text", max_nb_chars=32)

    class Meta:
        model = Building


class ConferenceRoomFactory(factory.django.DjangoModelFactory):
    name = factory.Faker("text", max_nb_chars=32)
    email = factory.Faker("email")
    building = factory.SubFactory(BuildingFactory)

    class Meta:
        model = ConferenceRoom
        django_get_or_create = ["email"]



import phonenumbers
from django.core.exceptions import ValidationError
from django.core.validators import URLValidator, validate_email
from django.utils.translation import gettext as _
from django.utils.translation import gettext_lazy, pgettext, pgettext_lazy
from dynamic_preferences.registries import global_preferences_registry
from rest_framework import serializers
from rest_framework.reverse import reverse
from schwifty import BIC, IBAN

from wbcore import serializers as wb_serializers
from wbcore.contrib.currency.serializers import CurrencyRepresentationSerializer
from wbcore.contrib.geography.serializers import GeographyRepresentationSerializer

from ..models import (
    AddressContact,
    BankingContact,
    EmailContact,
    SocialMediaContact,
    TelephoneContact,
    WebsiteContact,
)
from .entry_representations import EntryRepresentationSerializer


class AddressContactRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbcore:directory:addresscontact-detail")
    _detail_preview = wb_serializers.HyperlinkField(reverse_name="wbcore:directory:addresscontact-detail")

    geography_city = wb_serializers.CharField(max_length=255, read_only=True)
    label_key = "{{street}}, {{geography_city}}"

    class Meta:
        model = AddressContact
        fields = (
            "id",
            "street",
            "_detail",
            "_detail_preview",
            "geography_city",
        )


class CityRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbcore:directory:addresscontact-detail")
    _detail_preview = wb_serializers.HyperlinkField(reverse_name="wbcore:directory:addresscontact-detail")

    geography_city = wb_serializers.CharField(max_length=255, read_only=True)
    label_key = "{{geography_city}}"

    class Meta:
        model = AddressContact
        fields = (
            "id",
            "_detail",
            "_detail_preview",
            "geography_city",
        )


class BankingContactRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _entry = EntryRepresentationSerializer(source="entry")
    _detail = wb_serializers.HyperlinkField(reverse_name="wbcore:directory:bankingcontact-detail")

    class Meta:
        model = BankingContact
        fields = ("id", "_detail", "iban", "institute", "entry", "_entry", "status", "swift_bic")


class EmailContactRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbcore:directory:emailcontact-detail")

    def get_filter_params(self, request):
        if entry_id := request.parser_context["view"].kwargs.get("entry_id", None):
            return {"entry": entry_id}
        return dict()

    class Meta:
        model = EmailContact
        fields = ("id", "_detail", "address", "computed_str")


class TelephoneContactRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbcore:directory:telephonecontact-detail")

    class Meta:
        model = TelephoneContact
        fields = ("id", "_detail", "number", "primary")


class WebsiteContactRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbcore:directory:websitecontact-detail")

    class Meta:
        model = WebsiteContact
        fields = ("id", "_detail", "url")


class SocialMediaContactRepresentationSerializer(wb_serializers.RepresentationSerializer):
    _detail = wb_serializers.HyperlinkField(reverse_name="wbcore:directory:socialmediacontact-detail")

    class Meta:
        model = SocialMediaContact
        fields = ("id", "_detail")


class EmailContactSerializer(wb_serializers.ModelSerializer):
    company_contact = wb_serializers.BooleanField(read_only=True)
    _entry = EntryRepresentationSerializer(source="entry")

    def validate(self, data):
        address = data.get("address", None)
        entry = data.get("entry", None)
        if address and entry:
            email_contacts = EmailContact.objects.filter(address=address, entry=entry)
            if obj := self.instance:
                email_contacts = email_contacts.exclude(id=obj.id)
            if email_contacts.exists():
                raise serializers.ValidationError(
                    {
                        "address": pgettext(
                            "Warning for wrong email", "E-Mail address already in use for this {type}."
                        ).format(type=entry.entry_type.lower())
                    }
                )
        if address:
            try:
                validate_email(address)
            except ValidationError as e:
                raise serializers.ValidationError({"address": _("Invalid e-mail address")}) from e
        return data

    class Meta:
        model = EmailContact
        fields = (
            "id",
            "address",
            "company_contact",
            "entry",
            "_entry",
            "location",
            "primary",
        )


class WebsiteContactSerializer(wb_serializers.ModelSerializer):
    company_contact = wb_serializers.BooleanField(read_only=True)
    _entry = EntryRepresentationSerializer(source="entry")

    def validate(self, data):
        url = data.get("url", None)
        entry = data.get("entry", None)
        if url and entry:
            website_contact = WebsiteContact.objects.filter(url=url, entry=entry)
            if obj := self.instance:
                website_contact = website_contact.exclude(id=obj.id)
            if website_contact.exists():
                raise serializers.ValidationError(
                    {"url": _("Website already in use for this {type}.").format(type=entry.entry_type.lower())}
                )
        return data

    class Meta:
        model = WebsiteContact
        fields = ("id", "company_contact", "entry", "_entry", "location", "primary", "url")


class AddressContactSerializer(wb_serializers.ModelSerializer):
    company_contact = wb_serializers.BooleanField(read_only=True)
    _entry = EntryRepresentationSerializer(source="entry")
    _geography_city = GeographyRepresentationSerializer(source="geography_city", filter_params={"level": 3})
    geography_state = wb_serializers.CharField(read_only=True, label=gettext_lazy("State"))
    geography_country = wb_serializers.CharField(read_only=True, label=gettext_lazy("Country"))
    geography_continent = wb_serializers.CharField(read_only=True, label=gettext_lazy("Continent"))

    def validate(self, data):
        entry = data.get("entry", None)
        street = data.get("street", None)
        street_additional = data.get("street_additional", None)
        zip = data.get("zip", None)
        geography_city = data.get("geography_city", None)

        if entry and street and street_additional and zip and geography_city:
            address_contact = AddressContact.objects.filter(
                street=street, street_additional=street_additional, zip=zip, geography_city=geography_city, entry=entry
            )
            if obj := self.instance:
                address_contact = address_contact.exclude(id=obj.id)
            if address_contact.exists():
                raise serializers.ValidationError(
                    {
                        "non_field_errors": _("Address already in use for this {type}.").format(
                            type=entry.entry_type.lower()
                        )
                    }
                )
        return data

    @wb_serializers.register_resource()
    def entry_address(self, instance, request, user):
        if entry := instance.entry:
            return {
                "addresses": reverse("wbcore:directory:entry-addresscontact-list", args=[entry.id], request=request),
            }

        return {}

    class Meta:
        model = AddressContact
        fields = (
            "id",
            "entry",
            "_entry",
            "primary",
            "location",
            "street",
            "street_additional",
            "zip",
            "province",
            "company_contact",
            "_additional_resources",
            "geography_city",
            "_geography_city",
            "geography_state",
            "geography_country",
            "geography_continent",
        )


class TelephoneContactSerializer(wb_serializers.ModelSerializer):
    company_contact = wb_serializers.BooleanField(read_only=True)
    _entry = EntryRepresentationSerializer(source="entry")
    number = wb_serializers.TelephoneField(label=pgettext_lazy("Phonenumber", "Number"))

    def validate(self, data):
        number = data.get("number", None)
        entry = data.get("entry", None)
        formatting_successful = False
        if number:
            if entry:
                telephone_contact = TelephoneContact.objects.filter(number=number, entry=entry)
                if obj := self.instance:
                    telephone_contact = telephone_contact.exclude(id=obj.id)
                if telephone_contact.exists():
                    raise serializers.ValidationError(
                        {
                            "number": _("Phone number already in use for this {type}.").format(
                                type=entry.entry_type.lower()
                            )
                        }
                    )
            try:
                if number.startswith("00"):
                    number = number.replace("00", "+", 1)
                parser_number = phonenumbers.parse(
                    number, global_preferences_registry.manager()["directory__main_country_code"]
                )
                if parser_number:
                    formatted_number = phonenumbers.format_number(parser_number, phonenumbers.PhoneNumberFormat.E164)
                    formatting_successful = True
                    data["number"] = formatted_number
                else:
                    raise serializers.ValidationError({"number": _("Invalid phone number format")})
            except Exception as e:
                raise serializers.ValidationError({"number": _("Invalid phone number format")}) from e

            if entry and formatting_successful:
                telephone_contact = TelephoneContact.objects.filter(number=formatted_number, entry=entry)
                if obj := self.instance:
                    telephone_contact = telephone_contact.exclude(id=obj.id)
                if telephone_contact.exists():
                    raise serializers.ValidationError(
                        {
                            "number": _("Phone number already in use for this {entry}.").format(
                                type=entry.entry_type.lower()
                            )
                        }
                    )
        return data

    class Meta:
        model = TelephoneContact
        required_fields = ("entry",)
        fields = (
            "id",
            "company_contact",
            "entry",
            "_entry",
            "location",
            "number",
            "primary",
            "telephone_type",
            "_additional_resources",
        )


class BankingContactSerializer(wb_serializers.ModelSerializer):
    company_contact = wb_serializers.BooleanField(read_only=True, default=False)
    _currency = CurrencyRepresentationSerializer(source="currency")
    _entry = EntryRepresentationSerializer(source="entry")

    def validate(self, data):
        iban_repr = data.get("iban", None)
        swift_bic_repr = data.get("swift_bic", None)
        entry = data.get("entry", None)

        if iban_repr and iban_repr not in ["", "null"]:
            try:
                iban = IBAN(iban_repr)
                iban_formatted = iban.formatted
                data["iban"] = iban_formatted
            except ValueError as e:
                raise serializers.ValidationError({"iban": e}) from e

            if entry and iban_formatted:
                banking_contact = BankingContact.objects.filter(iban=iban_formatted, entry=entry)
                if obj := self.instance:
                    banking_contact = banking_contact.exclude(id=obj.id)
                if banking_contact.exists():
                    raise serializers.ValidationError(
                        {"iban": _("IBAN already in use for this {type}.").format(type=entry.entry_type.lower())}
                    )

        if swift_bic_repr and swift_bic_repr not in ["", "null"]:
            try:
                BIC(swift_bic_repr)
            except ValueError as e:
                raise serializers.ValidationError({"swift_bic": e}) from e
        return data

    class Meta:
        model = BankingContact
        read_only_fields = ("edited",)
        fields = (
            "id",
            "additional_information",
            "company_contact",
            "currency",
            "_currency",
            "edited",
            "entry",
            "_entry",
            "iban",
            "institute",
            "institute_additional",
            "location",
            "primary",
            "status",
            "swift_bic",
            "_additional_resources",
        )


class ReadOnlyBankingContactSerializer(BankingContactSerializer):
    class Meta(BankingContactSerializer.Meta):
        read_only_fields = BankingContactSerializer.Meta.fields


class SocialMediaContactSerializer(wb_serializers.ModelSerializer):
    company_contact = wb_serializers.BooleanField(read_only=True)
    _entry = EntryRepresentationSerializer(source="entry")

    def validate(self, data):
        url = data.get("url", self.instance.url if self.instance else "")
        try:
            URLValidator()(url)
        except ValidationError as e:
            raise serializers.ValidationError({"url": _("The URL you provided seems to be wrong.")}) from e

        if self.instance and (
            ((data_entry := data.get("entry")) != self.instance.entry)
            or ((data_url := data.get("url")) != self.instance.url)
        ):
            if SocialMediaContact.objects.filter(entry=data_entry, url=data_url).exists():
                raise serializers.ValidationError(
                    {"url": _("This social media account is already in use for this user")}
                )

        return super().validate(data)

    @wb_serializers.register_resource()
    def entry_social_media(self, instance, request, user):
        if entry := instance.entry:
            return {
                "social_media": reverse(
                    "wbcore:directory:entry-socialmediacontact-list", args=[entry.id], request=request
                ),
            }
        return {}

    class Meta:
        model = SocialMediaContact
        required_fields = ("entry", "platform", "url")
        fields = (
            "id",
            "company_contact",
            "entry",
            "_entry",
            "platform",
            "location",
            "primary",
            "url",
            "_additional_resources",
        )



from datetime import timedelta
from io import StringIO

from django.db.models import Case, Q, QuerySet, Value, When
from django.http import HttpResponse
from django.utils import timezone
from django.utils.functional import cached_property
from ics import Calendar
from psycopg.types.range import TimestamptzRange
from rest_framework import status
from rest_framework.decorators import (
    api_view,
    authentication_classes,
    permission_classes,
)
from rest_framework.permissions import AllowAny

from wbcore import viewsets
from wbcore.contrib.authentication.authentication import QueryTokenAuthentication
from wbcore.contrib.directory.models import Person
from wbcore.utils.itertools import get_inheriting_subclasses

from ..filters import CalendarItemFilter
from ..models import CalendarItem
from ..serializers import (
    CalendarItemModelSerializer,
    CalendarItemRepresentationSerializer,
)
from ..signals import draggable_calendar_item_ids
from .display import CalendarItemDisplay
from .endpoints import CalendarItemEndpointConfig
from .titles import CalendarItemTitleConfig


class CalendarItemRepresentationViewSet(viewsets.RepresentationViewSet):
    queryset = CalendarItem.objects.all()
    serializer_class = CalendarItemRepresentationSerializer


class CalendarItemMetaClass(type(viewsets.ModelViewSet)):
    def __new__(cls, *args, **kwargs):
        _class = super().__new__(cls, *args, **kwargs)
        dependant_identifiers = []
        for subclass in get_inheriting_subclasses(_class):
            if identifier := getattr(subclass, "IDENTIFIER", None):
                dependant_identifiers.append(identifier)
        _class.DEPENDANT_IDENTIFIERS = dependant_identifiers
        return _class


class CalendarItemViewSet(viewsets.ModelViewSet, metaclass=CalendarItemMetaClass):
    display_config_class = CalendarItemDisplay
    endpoint_config_class = CalendarItemEndpointConfig
    filterset_class = CalendarItemFilter
    search_fields = ("title",)
    serializer_class = CalendarItemModelSerializer
    title_config_class = CalendarItemTitleConfig
    ordering = ("period__startswith", "id")
    queryset = CalendarItem.objects.all()

    @cached_property
    def request_profile(self):
        return getattr(self.request.user, "profile", None)

    @cached_property
    def draggable_calendar_item_ids(self) -> QuerySet[CalendarItem]:
        """
        Property to store the unioned queryset of draggable calendar items. Each module inheriting from CalendarItem are expected to define the signal receiver
        """
        qs = CalendarItem.objects.none()
        for _, union_qs in draggable_calendar_item_ids.send(CalendarItem, request=self.request):
            qs = qs.union(union_qs)
        return qs

    @cached_property
    def allowed_privates_calendar_item_ids(self) -> list[int]:
        if self.request_profile:
            return list(
                CalendarItem.objects.filter(
                    entities=self.request_profile, visibility=CalendarItem.Visibility.PRIVATE
                ).values_list("id", flat=True)
            )
        return list()

    @cached_property
    def has_user_administrate_permission(self) -> bool:
        return CalendarItem.has_user_administrate_permission(self.request.user)

    def get_queryset(self):
        qs = (
            super()
            .get_queryset()
            .annotate(
                is_draggable=Case(When(id__in=self.draggable_calendar_item_ids, then=True), default=False),
                is_private=Case(
                    When(
                        Q(visibility=CalendarItem.Visibility.PRIVATE)
                        & ~Q(id__in=self.allowed_privates_calendar_item_ids),
                        then=True,
                    ),
                    default=False,
                ),
            )
        )
        if self.has_user_administrate_permission:
            qs = qs.annotate(is_confidential=Value(False))
        else:
            qs = qs.annotate(
                is_confidential=Case(
                    When(visibility=CalendarItem.Visibility.CONFIDENTIAL, then=True),
                    default=False,
                )
            )
        return qs.prefetch_related("entities")


class InfiniteCalendarItemViewSet(CalendarItemViewSet):
    pagination_class = None


class OwnCalendarItemViewSet(CalendarItemViewSet):
    def get_queryset(self):
        if self.request_profile:
            return super().get_queryset().filter(is_cancelled=False, entities=self.request_profile)
        return super().get_queryset().none()


@api_view(["GET"])
@permission_classes([AllowAny])
@authentication_classes([QueryTokenAuthentication])
def get_ics(request):
    """
    API Calendar ICS request endpoint

    Return the ICS formatted calendar for the user with token
    """
    conference_room = request.GET.get("conference_room", "False") == "True"
    team = request.GET.get("team", "False") == "True"
    only_today = request.GET.get("only_today", "False") == "True"
    if profile := getattr(request.user, "profile", None):
        qs = CalendarItem.objects.exclude(is_cancelled=False)
        if conference_room:
            qs = qs.filter(conference_room=True).distinct()
        elif team:
            qs = CalendarItem.objects.filter(entities__in=Person.objects.filter_only_internal().values("id"))
        else:
            qs = CalendarItem.objects.filter(entities__id=profile.id)
        if only_today:
            minimum_date = timezone.now().date()
            maximum_date = minimum_date + timedelta(days=1)
        else:
            minimum_date = timezone.now() - timedelta(days=30)
            maximum_date = timezone.now() + timedelta(days=30)
        min_max_range = TimestamptzRange(minimum_date, maximum_date)
        gen = qs.filter(period__contained_by=min_max_range)

        ical = Calendar()
        for occurence in gen:
            start = occurence.period.lower
            end = occurence.period.upper
            activity = occurence
            event = activity.to_ics(start, end)
            if event:
                ical.events.add(event)
        data = StringIO(str(ical))
        response = HttpResponse(data, content_type="text/calendar")
        response["Content-Disposition"] = f'attachment; filename="{profile.computed_str}.ics"'
        return response
    return HttpResponse(status=status.HTTP_400_BAD_REQUEST)



import warnings


def deprecate_warning(message: str):
    warnings.simplefilter("always", DeprecationWarning)
    warnings.warn(
        message,
        category=DeprecationWarning,
        stacklevel=2,
    )
    warnings.simplefilter("default", DeprecationWarning)



from django.utils.translation import gettext as _

from wbcore.contrib.icons import WBIcon
from wbcore.metadata.configs import buttons as bt
from wbcore.metadata.configs.display.instance_display.shortcuts import (
    Display,
    create_simple_display,
)
from wbcore.metadata.configs.preview import PreviewViewConfig


class AddressPreviewConfig(PreviewViewConfig):
    def get_display(self) -> Display:
        return create_simple_display([["street", "zip", "geography_city"]])

    def get_buttons(self):
        return [
            bt.WidgetButton(key="addresses", icon=WBIcon.LOCATION.icon, label=_("Show all addresses")),
        ]



from .menu import TAG_MENUITEM
from .viewsets import (
    TagGroupModelSerializer,
    TagGroupRepresentationSerializer,
    TagModelViewSet,
    TagGroupModelViewSet,
    TagGroupRepresentationViewSet,
    TagRepresentationViewSet,
    TagTagGroupModelViewSet,
)



from anymail.signals import inbound
from django.apps import AppConfig
from django.apps import apps as global_apps
from django.db import DEFAULT_DB_ALIAS
from django.db.models.signals import post_migrate
from django.utils.module_loading import autodiscover_modules


class ImportExportConfig(AppConfig):
    name = "wbcore.contrib.io"

    def ready(self):
        """
        registered source from settings
        """
        # Implicitly connect a signal handlers decorated with @receiver.
        from wbcore.contrib.io.management import load_sources_from_settings

        from .backends.mail import handle_inbound

        # Explicitly connect a signal handler.
        inbound.connect(handle_inbound)

        def autodiscover_backends(
            app_config, verbosity=2, interactive=True, using=DEFAULT_DB_ALIAS, apps=global_apps, **kwargs
        ):
            # we wrap the autodiscover into a post_migrate receiver because we expect db calls
            autodiscover_modules("import_export.backends")

        post_migrate.connect(
            autodiscover_backends,
            dispatch_uid="wbcore.io.autodiscover_backends",
        )
        post_migrate.connect(
            load_sources_from_settings,
            dispatch_uid="wbcore.contrib.io.load_sources_from_settings",
        )



from typing import Callable

from django.conf import settings

from ..signals.instance_buttons import add_extra_button
from .buttons import add_clear_cache_button
from .registry import periodic_cache_registry


def cache_table(
    timeout: int | Callable | None = None,
    key_prefix: str | Callable | None = None,
    periodic_caching: bool = False,
    periodic_caching_view_kwargs: list[dict[str, str]] | Callable | None = None,
    periodic_caching_get_parameters: list[dict[str, str]] | Callable | None = None,
):
    def _decorator(pandas_view_class):
        pandas_view_class.CACHE_ENABLED = not settings.DEBUG
        pandas_view_class.CACHE_TIMEOUT = timeout
        pandas_view_class.CACHE_KEY_PREFIX = key_prefix
        add_extra_button.connect(add_clear_cache_button, sender=pandas_view_class, weak=False)

        if periodic_caching:
            periodic_cache_registry.add(
                pandas_view_class,
                view_kwargs=periodic_caching_view_kwargs,
                get_parameters=periodic_caching_get_parameters,
            )
        return pandas_view_class

    return _decorator



from wbcore.metadata.configs.endpoints import EndpointViewConfig


class UserActivityModelEndpointConfig(EndpointViewConfig):
    def get_endpoint(self, **kwargs):
        return None


class UserActivityUserModelEndpointConfig(UserActivityModelEndpointConfig):
    pass


class UserActivityTableEndpointConfig(UserActivityModelEndpointConfig):
    pass



from typing import TYPE_CHECKING

from wbcore.metadata.configs import display as dp

if TYPE_CHECKING:
    from wbcore.contrib.guardian.viewsets import PivotUserObjectPermissionModelViewSet


class PivotUserObjectPermissionDisplayViewConfig(dp.DisplayViewConfig):
    view: "PivotUserObjectPermissionModelViewSet"

    def get_list_display(self) -> dp.ListDisplay:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="userref", label="User", width=250),
                *[
                    dp.Field(
                        key=permission.codename,
                        label=permission.name,
                        width=150,
                        formatting_rules=[
                            dp.FormattingRule(style={"backgroundColor": "#FF6961"}, condition=("==", False)),
                            dp.FormattingRule(style={"backgroundColor": "#77DD77"}, condition=("==", True)),
                        ],
                    )
                    for permission in self.view.permissions
                ],
            ],
            formatting=[
                dp.Formatting(
                    column="non_editable",
                    formatting_rules=[dp.FormattingRule(style={"color": "gray"}, condition=("==", True))],
                )
            ],
        )

    def get_instance_display(self) -> dp.Display:
        gta = [["userref"]]

        for permission in self.view.permissions:
            gta.append([permission.codename])

        return dp.create_simple_display(grid_template_areas=gta)



import magic
from django.apps import apps
from django.conf import settings
from import_export.formats.base_formats import CSV
from import_export.results import RowResult
from import_export.signals import post_import
from wbcore.utils.importlib import import_from_dotted_path


def default_import_parse(import_source):
    resource_path = import_source.resource_kwargs["resource_path"]
    resource_kwargs = import_source.resource_kwargs["resource_kwargs"]
    resource_class = import_from_dotted_path(resource_path)
    resource = resource_class(**resource_kwargs)
    input_format = CSV(encoding="utf-8-sig")

    file_stream = import_source.file.read()
    if input_format.CONTENT_TYPE == magic.from_buffer(file_stream, mime=True):
        dataset = input_format.create_dataset(file_stream)

        model = apps.get_model(import_source.parser_handler.handler)
        result = resource.import_data(
            dataset,
            dry_run=False,
            file_name=import_source.file.name,
            user=import_source.creator,
            rollback_on_validation_errors=True,
            raise_errors=settings.DEBUG,
        )
        import_source.file.close()
        post_import.send(sender=None, model=model)
        success_message = """
        {} import finished:
        * new {}
        * updated {}
        * skipped {}
        * failed {}
        * deleted {}
        * invalid {}
        """.format(
            model._meta.verbose_name_plural,
            result.totals[RowResult.IMPORT_TYPE_NEW],
            result.totals[RowResult.IMPORT_TYPE_UPDATE],
            result.totals[RowResult.IMPORT_TYPE_SKIP],
            result.totals[RowResult.IMPORT_TYPE_ERROR],
            result.totals[RowResult.IMPORT_TYPE_DELETE],
            result.totals[RowResult.IMPORT_TYPE_INVALID],
        )
        import_source.log = success_message
        import_source.save()



from wbcore import viewsets
from wbcore.contrib.workflow.filters import ConditionFilter
from wbcore.contrib.workflow.models import Condition
from wbcore.contrib.workflow.serializers import ConditionModelSerializer
from wbcore.contrib.workflow.viewsets.display import ConditionDisplayConfig
from wbcore.contrib.workflow.viewsets.endpoints import ConditionEndpointConfig
from wbcore.contrib.workflow.viewsets.titles import ConditionTitleConfig


class ConditionModelViewSet(viewsets.ModelViewSet):
    LIST_DOCUMENTATION = "workflow/markdown/documentation/condition.md"
    queryset = Condition.objects.all()
    serializer_class = ConditionModelSerializer
    display_config_class = ConditionDisplayConfig
    title_config_class = ConditionTitleConfig
    search_fields = ("attribute_name", "expected_value")
    ordering = ("transition", "attribute_name", "operator", "negate_operator", "expected_value")
    filterset_class = ConditionFilter
    endpoint_config_class = ConditionEndpointConfig
    ordering_fields = (
        "attribute_name",
        "transition__name",
        "operator",
        "expected_value",
        "negate_operator",
    )

    def get_queryset(self):
        queryset = super().get_queryset().select_related("transition")
        if transition_id := self.kwargs.get("transition_id"):
            return queryset.filter(transition=transition_id)
        return queryset



import pytest
from pytest_mock import MockerFixture

from wbcore.contrib.workflow.dispatch import check_workflow_for_instance
from wbcore.contrib.workflow.models import Process, ProcessStep, Step, Workflow


class TestDispatch:
    @pytest.fixture
    def mocked_instance(self, mocker: MockerFixture):
        return mocker.MagicMock()

    # Fixture that automatically patches ContentType lookup in every test.
    @pytest.fixture(autouse=True)
    def patch_content_type(self, mocker: MockerFixture):
        return mocker.patch(
            "wbcore.contrib.workflow.dispatch.ContentType.objects.get_for_model",
            return_value="dummy_content_type",
        )

    # Fixture for creating a fake process step with a mismatching status.
    @pytest.fixture
    def mocked_process_step(self, mocker: MockerFixture):
        # Create the fake casted step.
        mocked_casted_step = mocker.MagicMock(spec=Step)

        # Create a fake workflow with a status field.
        mocked_workflow = mocker.MagicMock(spec=Workflow)
        mocked_workflow.status_field = "status_a"

        # Create a fake process that uses the workflow.
        mocked_process = mocker.MagicMock(spec=Process)
        mocked_process.workflow = mocked_workflow

        # Create a fake process step with a mismatching status.
        mocked_process_step = mocker.MagicMock(spec=ProcessStep)
        mocked_process_step.process = mocked_process
        mocked_process_step.status = "status_b"
        mocked_process_step.step.get_casted_step.return_value = mocked_casted_step

        return mocked_process_step, mocked_casted_step

    def test_check_workflow_for_instance_fail_process_steps(
        self, mocked_instance, mocked_process_step, mocker: MockerFixture
    ):
        # Arrange
        mocked_process_step, mocked_casted_step = mocked_process_step

        # Patch ProcessStep.objects.filter to return a fake process step.
        mocker.patch(
            "wbcore.contrib.workflow.models.ProcessStep.objects.filter",
            return_value=[mocked_process_step],
        )
        # Patch get_start_steps_for_instance to return an empty list.
        mocker.patch(
            "wbcore.contrib.workflow.models.Workflow.get_start_steps_for_instance",
            return_value=[],
        )

        # Act
        check_workflow_for_instance(sender=None, instance=mocked_instance, created=True)

        # Assert: Ensure set_failed was called with the correct arguments.
        mocked_casted_step.set_failed.assert_called_once_with(mocked_process_step, "Invalid status detected!")

    def test_check_workflow_for_instance_start_workflow(self, mocked_instance, mocker: MockerFixture):
        # Arrange: Create two fake start steps, each with its own workflow.
        fake_start_step1 = mocker.MagicMock()
        fake_workflow1 = mocker.MagicMock(spec=Workflow)
        fake_start_step1.workflow = fake_workflow1

        fake_start_step2 = mocker.MagicMock()
        fake_workflow2 = mocker.MagicMock(spec=Workflow)
        fake_start_step2.workflow = fake_workflow2

        # Patch get_start_steps_for_instance to return our fake start steps.
        get_start_steps_patch = mocker.patch(
            "wbcore.contrib.workflow.models.Workflow.get_start_steps_for_instance",
            return_value=[fake_start_step1, fake_start_step2],
        )
        # Patch ProcessStep.objects.filter to return an empty list.
        mocker.patch(
            "wbcore.contrib.workflow.models.ProcessStep.objects.filter",
            return_value=[],
        )

        # Act
        check_workflow_for_instance(sender=None, instance=mocked_instance, created=True)

        # Assert: Verify that get_start_steps_for_instance was called with our instance.
        get_start_steps_patch.assert_called_once_with(mocked_instance)
        # Verify that each fake workflow's start_workflow was called with its corresponding start step and the instance.
        fake_workflow1.start_workflow.assert_called_once_with(fake_start_step1, mocked_instance)
        fake_workflow2.start_workflow.assert_called_once_with(fake_start_step2, mocked_instance)



from django.contrib import admin

from .models import Tag, TagGroup


@admin.register(Tag)
class TagModelAdmin(admin.ModelAdmin):
    list_display = ("title", "slug", "color", "content_type")
    search_fields = ("title", "slug")
    readonly_fields = ("managed",)


@admin.register(TagGroup)
class TagGroupModelAdmin(admin.ModelAdmin):
    list_display = ("title",)
    search_fields = ("title",)
    readonly_fields = ("managed",)



from django.dispatch import Signal

add_additional_resource = Signal()  # "serializer", "instance", "request", "user"
add_instance_additional_resource = Signal()  # "serializer", "instance", "request", "user"
add_dynamic_button = Signal()  # "serializer", "instance", "request", "user"



from .user_activities import (
    UserActivityChartTitleConfig,
    UserActivityModelTitleConfig,
    UserActivityTableTitleConfig,
    UserActivityUserChartTitleConfig,
    UserActivityUserModelTitleConfig,
)
from .users import (
    UserModelTitleConfig,
    UserPermissionsModelTitleConfig,
    UserProfileModelTitleConfig,
)
